#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2009 Pâris Quentin
# Copyright (C) 2007-2010 PlayOnLinux Team

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import os
import shutil
import stat

import subprocess
import wx

import lib.Variables as Variables
import lib.playonlinux as playonlinux
from configurewindow.ConfigureWindowNotebook import ConfigureWindowNotebook

## TODO: Terminate refactoring this module
class ConfigureWindow(wx.Frame):
    def __init__(self,parent,id,title,shortcut, isPrefix = False):
        wx.Frame.__init__(self, parent, -1, title, size = (800, 455+Variables.windows_add_size))
        self.SetIcon(wx.Icon(Variables.playonlinux_env+"/etc/playonlinux.png", wx.BITMAP_TYPE_ANY))
        self.SetTitle(_('{0} configuration').format(os.environ["APPLICATION_TITLE"]))

        self.windowSplitter = wx.SplitterWindow(self, -1, style=wx.SP_NOBORDER)

        self.panelEmpty = wx.Panel(self.windowSplitter, -1)
        self.onglets = ConfigureWindowNotebook(self.windowSplitter)

        self.noselect = wx.StaticText(self.panelEmpty, -1, _('Please select a program or a virtual drive to configure'),pos=(0,150),style=wx.ALIGN_RIGHT)
        self.noselect.SetPosition(((600-self.noselect.GetSize()[0])/2,150))

        self.noselect.Wrap(600)
        if(isPrefix == True):
            self.onglets.s_isPrefix = True
            self.onglets.s_prefix = shortcut
        else:
            self.onglets.s_isPrefix = False
            self.onglets.s_title = shortcut

        self.images = wx.ImageList(16, 16)

        self.leftPanel = wx.Panel(self.windowSplitter, -1)
        self.leftPanelSizer = wx.BoxSizer(wx.VERTICAL)
        self.leftPanel.SetSizer(self.leftPanelSizer)

        self.list_game = wx.TreeCtrl(self.leftPanel, 900, size = wx.DefaultSize, style=wx.TR_HIDE_ROOT)
        self.leftPanelSizer.Add(self.list_game, 1, wx.EXPAND)

        self.prefixPanel = wx.Panel(self.leftPanel, -1)
        self.leftPanelSizer.Add(self.prefixPanel, 0, wx.EXPAND)

        if(os.environ["POL_OS"] == "Mac"):
            self.AddPrefix = wx.Button(self.prefixPanel, 1001, _("New"), pos=(0, -8), size=(93, 30))
            self.DelPrefix = wx.Button(self.prefixPanel, 1002, _("Remove"), pos=(98, -8), size=(93, 30))
        else:
            self.AddPrefix = wx.Button(self.prefixPanel, 1001, _("New"), pos=(0, 0), size=(95, 25))
            self.DelPrefix = wx.Button(self.prefixPanel, 1002, _("Remove"), pos=(100, 0), size=(95, 25))

        wx.EVT_BUTTON(self, 1001, self.NewPrefix)
        wx.EVT_BUTTON(self, 1002, self.DeletePrefix)


        self.list_game.SetSpacing(0)
        self.list_game.SetImageList(self.images)


        self.windowSplitter.SplitVertically(self.leftPanel, self.panelEmpty)
        self.windowSplitter.SetSashPosition(200)

        self.onglets.General(_("General"))
        self.onglets.Wine("Wine")
        self.onglets.Packages(_("Install components"))
        self.onglets.Display(_("Display"))
        self.onglets.Miscellaneous(_("Miscellaneous"))

        self.list_software()

        self.onglets.panelGeneral.Bind(wx.EVT_LEFT_UP, self.onglets.ReleaseTyping)
        wx.EVT_TREE_SEL_CHANGED(self, 900, self.change_program_to_selection)

        self.timer = wx.Timer(self, 1)
        self.Bind(wx.EVT_TIMER, self.AutoReload, self.timer)

        self.timer.Start(500)
        self.oldreload = None
        self.oldimg = None
        self.oldpref = None
        self.oldver32 = None
        self.olderver64 = None
        self.AutoReload(self)

    def NewPrefix(self, event):
        subprocess.Popen(["bash", Variables.playonlinux_env+"/bash/create_prefix"])

    def DeletePrefix(self, event):
        if(self.onglets.s_isPrefix == True):
            if(self.onglets.s_prefix == "default"):
                wx.MessageBox(_("This virtual drive is protected"), os.environ["APPLICATION_TITLE"])
            else:
                if(wx.YES == wx.MessageBox(_("Are you sure you want to delete {0} virtual drive ?").format(self.onglets.s_prefix.encode("utf-8","replace")).decode("utf-8","replace"), os.environ["APPLICATION_TITLE"], style=wx.YES_NO | wx.ICON_QUESTION)):
                    mylist = os.listdir(Variables.playonlinux_rep+"/shortcuts")
                    for element in mylist:
                        if(playonlinux.getPrefix(element).lower() == self.onglets.s_prefix.lower()):
                            subprocess.Popen(["bash", Variables.playonlinux_env+"/bash/uninstall", "--non-interactive", element])
                    self._delete_directory(Variables.playonlinux_rep+"/wineprefix/"+self.onglets.s_prefix)
        else:
            if(wx.YES == wx.MessageBox(_("Are you sure you want to delete {0} ?").format(self.onglets.s_title.encode("utf-8","replace")).decode("utf-8","replace"), os.environ["APPLICATION_TITLE"], style=wx.YES_NO | wx.ICON_QUESTION)):
                subprocess.Popen(["bash", Variables.playonlinux_env+"/bash/uninstall", "--non-interactive", self.onglets.s_title.encode('utf-8', 'replace')])

        self.onglets.s_isPrefix = True
        self.change_program("default",True)
        self.list_game.SelectItem(self.prefixes_item["default"])

    def _delete_directory(self, root_path):
        """
        Remove a directory tree, making sure no directory rights get in the way.
        It assumes everything is owned by the user however.
        """

        # Handle symlink
        if os.path.islink(root_path):
            os.remove(root_path)
            # Shall we warn the user that the target prefix has not been cleared?
        else:
            # need exec right to dereference content
            # need read right to list content
            # need write right to remove content
            needed_dir_rights = stat.S_IXUSR|stat.S_IRUSR|stat.S_IWUSR

            # topdown=True, the default, is necessary to fix directories rights
            # before trying to list them
            for dirname, dirs, files in os.walk(root_path):
                for dir in dirs:
                    fullpath = os.path.join(dirname, dir)
                    # To speed up the process, only modify metadata when necessary
                    attr = os.lstat(fullpath)
                    if attr.st_mode & needed_dir_rights != needed_dir_rights:
                        print "%s rights need fixing" % fullpath
                        os.chmod(fullpath, needed_dir_rights)

            # Alright, now we should be able to proceed
            shutil.rmtree(root_path)

    def AutoReload(self, event):
        if(self.onglets.typing == False):
            reload = os.path.getmtime(Variables.playonlinux_rep+"/shortcuts")
            if(reload != self.oldreload):
                self.list_software()
                self.oldreload = reload

            reloadimg = os.path.getmtime(Variables.playonlinux_rep+"/icones/32")
            if(reloadimg != self.oldimg):
                self.list_software()
                self.oldimg = reloadimg

            reloadpref = os.path.getmtime(Variables.playonlinux_rep+"/wineprefix")
            if(reloadpref != self.oldpref):
                self.list_software()
                self.oldpref = reloadpref

            reloadver32 = os.path.getmtime(Variables.playonlinux_rep+"/wine/"+Variables.os_name+"-x86/")
            reloadver64 = os.path.getmtime(Variables.playonlinux_rep+"/wine/"+Variables.os_name+"-amd64/")

            if(reloadver32 != self.oldver32 or reloadver64 != self.oldver64):
                self.oldver32 = reloadver32
                self.oldver64 = reloadver64
                self.onglets.UpdateVersions(self.onglets.arch)

    def change_program_to_selection(self, event):
        parent =  self.list_game.GetItemText(self.list_game.GetItemParent(self.list_game.GetSelection()))
        self.current_sel = self.list_game.GetItemText(self.list_game.GetSelection())

        if(parent == "#ROOT#"):
            self.onglets.s_isPrefix = True
        else:
            self.onglets.s_isPrefix = False

        self.change_program(self.current_sel,self.onglets.s_isPrefix)

    def change_program(self, new_prgm,isPrefix = False):
        self.onglets.changing_selection = True
        if(isPrefix == True):
            self.onglets.s_isPrefix = True
            try:
                if(self.current_sel == "default"):
                    self.windowSplitter.Unsplit()
                    self.windowSplitter.SplitVertically(self.leftPanel, self.panelEmpty)
                    self.windowSplitter.SetSashPosition(200)
                else:
                    self.windowSplitter.Unsplit()
                    self.windowSplitter.SplitVertically(self.leftPanel, self.onglets)
                    self.windowSplitter.SetSashPosition(200)
            except AttributeError:
                self.windowSplitter.Unsplit()
                self.windowSplitter.SplitVertically(self.leftPanel, self.panelEmpty)
                self.windowSplitter.SetSashPosition(200)
        else:
            self.windowSplitter.Unsplit()
            self.windowSplitter.SplitVertically(self.leftPanel, self.onglets)
            self.windowSplitter.SetSashPosition(200)
        self.onglets.UpdateValues(new_prgm)
        self.Refresh()
        self.SetFocus()
        try:
            self.GetTopWindow().Raise()
        except:
            pass

    def list_software(self):
        self.games = os.listdir(Variables.playonlinux_rep+"shortcuts/")
        self.games.sort()

        self.prefixes = os.listdir(Variables.playonlinux_rep+"wineprefix/")
        self.prefixes.sort()

        self.games_item = {}
        self.prefixes_item = {}

        try:
            self.games.remove(".DS_Store")
        except:
            pass

        try:
            self.prefixes.remove(".DS_Store")
        except:
            pass

        self.list_game.DeleteAllItems()
        self.images.RemoveAll()
        root = self.list_game.AddRoot("#ROOT#")

        self.i = 0
        for prefix in self.prefixes:
            if(os.path.isdir(Variables.playonlinux_rep+"wineprefix/"+prefix)):
                self.prefixes_item[prefix] = self.list_game.AppendItem(root, prefix, self.i)

                if(os.path.exists(Variables.playonlinux_rep+"/wineprefix/"+prefix+"/icon")):
                    self.file_icone = Variables.playonlinux_rep+"/wineprefix/"+prefix+"/icon"
                else:
                    try:
                        archdd = playonlinux.GetSettings('ARCH',prefix)
                        if(archdd == "amd64"):
                            archdd = "64"
                        else:
                            archdd = "32"
                    except:
                        archdd = "32"
                    self.file_icone = Variables.playonlinux_env+"/resources/images/menu/virtual_drive_"+archdd+".png"

                try:
                    self.bitmap = wx.Image(self.file_icone)
                    self.bitmap.Rescale(16,16,wx.IMAGE_QUALITY_HIGH)
                    self.bitmap = self.bitmap.ConvertToBitmap()
                    self.images.Add(self.bitmap)
                except:
                    pass

                self.list_game.SetItemBold(self.prefixes_item[prefix], True)

                for game in self.games: #METTRE EN 32x32
                    if(playonlinux.getPrefix(game).lower() == prefix.lower()):
                        if(os.path.exists(Variables.playonlinux_rep+"/icones/32/"+game)):
                            self.file_icone = Variables.playonlinux_rep+"/icones/32/"+game
                        else:
                            self.file_icone = Variables.playonlinux_env+"/etc/playonlinux32.png"

                        try:
                            self.bitmap = wx.Image(self.file_icone)
                            self.bitmap.Rescale(16,16,wx.IMAGE_QUALITY_HIGH)
                            self.bitmap = self.bitmap.ConvertToBitmap()
                            self.images.Add(self.bitmap)
                        except:
                            pass
                        self.i += 1
                        self.games_item[game] = self.list_game.AppendItem(self.prefixes_item[prefix], game, self.i)

                self.i += 1

        self.list_game.ExpandAll()
        try:
            if(self.onglets.s_isPrefix == True):
                self.list_game.SelectItem(self.prefixes_item[self.onglets.s_prefix.encode("utf-8","replace")])
            else:
                self.list_game.SelectItem(self.games_item[self.onglets.s_title.encode("utf-8","replace")])
        except:
            self.onglets.s_isPrefix = True
            self.change_program("default",True)
            self.list_game.SelectItem(self.prefixes_item["default"])

    def app_Close(self, event):
        self.Destroy()

    def apply_settings(self, event):
        self.Destroy()



import os
import shlex
import subprocess

import wx

from configurewindow.PackageList import PackageList
from lib import playonlinux, Variables, wine


class ConfigureWindowNotebook(wx.Notebook):
    def __init__(self, parent):
        self.packageList = PackageList()
        self.notebook = wx.Notebook.__init__(self, parent, -1)
        self.typing = False
        self.changing_selection = False

    def ChangeTitle(self, new_title):
        self.s_title = new_title
        self.s_prefix = playonlinux.getPrefix(self.s_title)
        self.changing_selection = True
        self.general_elements["name"].SetValue(new_title)
        self.changing = True

    def winebash(self, command, new_env=None):
        args = shlex.split(command.encode("utf-8", "replace"))
        if (self.s_isPrefix == True):
            subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/winebash", "--prefix",
                              self.s_prefix.encode('utf-8', 'replace')] + args, env=new_env)
        else:
            subprocess.Popen(
                ["bash", Variables.playonlinux_env + "/bash/winebash", self.s_title.encode('utf-8', 'replace')] + args,
                env=new_env)

    def evt_winecfg(self, event):
        self.winebash("winecfg")

    def evt_regedit(self, event):
        self.winebash("regedit")

    def evt_cmd(self, event):
        # http://bugs.winehq.org/show_bug.cgi?id=10063
        new_env = os.environ
        new_env["LANG"] = "C"

        self.winebash("wineconsole cmd", new_env)

    def evt_taskmgr(self, event):
        self.winebash("taskmgr")

    def evt_rep(self, event):
        try:
            os.remove(os.environ["POL_USER_ROOT"] + "/wineprefix/" + self.s_prefix + "/.update-timestamp")
        except:
            pass
        self.winebash("wineboot")

    def evt_wineboot(self, event):
        self.winebash("wineboot")

    def evt_killall(self, event):
        self.winebash("wineserver -k")

    def evt_control(self, event):
        self.winebash("control")

    def evt_config(self, event):
        subprocess.Popen(["bash", Variables.playonlinux_rep + "/configurations/configurators/" + self.s_title])

    def install_package(self, event):
        selectedPackageName = self.Menu.GetItemText(self.Menu.GetSelection())
        if selectedPackageName:
            selectedPackage = self.packageList.getPackageFromName(selectedPackageName)

            if self.s_isPrefix == False:
                subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/installpolpackages",
                                  self.s_title.encode('utf-8', 'replace'), selectedPackage])
            else:
                subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/installpolpackages", "--prefix",
                                  self.s_prefix.encode('utf-8', 'replace'), selectedPackage])

    def AddGeneralChamp(self, title, shortname, value, num):
        self.general_elements[shortname + "_text"] = wx.StaticText(self.panelGeneral, -1, title,
                                                                   pos=(15, 19 + num * 40))
        self.general_elements[shortname] = wx.TextCtrl(self.panelGeneral, 200 + num, value, pos=(300, 23 + num * 40),
                                                       size=(250, 20))

    #       self.general_elements[shortname].SetValue(value)

    def AddGeneralElement(self, title, shortname, elements, wine, num):
        if (shortname == "wineversion"):
            elements.insert(0, "System")
            wine.insert(0, "System")
            elemsize = (225, 25)
        else:
            elemsize = (250, 25)

        self.general_elements[shortname + "_text"] = wx.StaticText(self.panelGeneral, -1, title,
                                                                   pos=(15, 19 + num * 40))

        self.general_elements[shortname] = wx.ComboBox(self.panelGeneral, 200 + num, style=wx.CB_READONLY,
                                                       pos=(300, 17 + num * 40), size=elemsize)
        self.general_elements[shortname].AppendItems(elements)
        self.general_elements[shortname].SetValue(elements[0])

        if (shortname == "wineversion"):
            self.addBitmap = wx.Image(Variables.playonlinux_env + "/resources/images/icones/list-add.png",
                                      wx.BITMAP_TYPE_ANY).ConvertToBitmap()
            if (os.environ["POL_OS"] == "Linux" or os.environ["POL_OS"] == "FreeBSD"):
                self.general_elements["wineversion_button"] = wx.BitmapButton(self.panelGeneral, 601,
                                                                              pos=(527, 19 + num * 40), size=(21, 21),
                                                                              bitmap=self.addBitmap)
            if (os.environ["POL_OS"] == "Mac"):
                self.general_elements["wineversion_button"] = wx.BitmapButton(self.panelGeneral, 601,
                                                                              pos=(522, 15 + num * 40), size=(21, 21),
                                                                              bitmap=self.addBitmap)

    def General(self, nom):
        self.panelGeneral = wx.Panel(self, -1)
        self.AddPage(self.panelGeneral, nom)
        self.general_elements = {}
        # Les polices
        if (os.environ["POL_OS"] == "Mac"):
            self.fontTitle = wx.Font(14, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD, False, "",
                                     wx.FONTENCODING_DEFAULT)
            self.caption_font = wx.Font(11, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL, False, "",
                                        wx.FONTENCODING_DEFAULT)
        else:
            self.fontTitle = wx.Font(12, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD, False, "",
                                     wx.FONTENCODING_DEFAULT)
            self.caption_font = wx.Font(8, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL, False, "",
                                        wx.FONTENCODING_DEFAULT)

        self.txtGeneral = wx.StaticText(self.panelGeneral, -1, _("General"), (10, 10), wx.DefaultSize)
        self.txtGeneral.SetFont(self.fontTitle)

        self.AddGeneralButton(_("Make a new shortcut from this virtual drive"), "newshort", 1)
        self.AddGeneralChamp(_("Name"), "name", "", 2)
        self.AddGeneralElement(_("Wine version"), "wineversion", [], [], 3)
        self.AddGeneralChamp(_("Debug flags"), "winedebug", "", 4)

        self.AddGeneralElement(_("Virtual drive"), "wineprefix", playonlinux.Get_Drives(), playonlinux.Get_Drives(), 5)

        self.AddGeneralChamp(_("Arguments"), "arguments", "", 6)

        self.configurator_title = wx.StaticText(self.panelGeneral, -1, "", (10, 294), wx.DefaultSize)
        self.configurator_title.SetFont(self.fontTitle)
        self.configurator_button = wx.Button(self.panelGeneral, 106, _("Run configuration wizard"), pos=(15, 324))

        wx.EVT_TEXT(self, 202, self.setname)
        wx.EVT_TEXT(self, 206, self.setargs)
        wx.EVT_TEXT(self, 204, self.setwinedebug)

        wx.EVT_COMBOBOX(self, 203, self.assign)
        wx.EVT_COMBOBOX(self, 205, self.assignPrefix)
        wx.EVT_BUTTON(self, 601, self.Parent.Parent.Parent.WineVersion)

    def Wine(self, nom):
        self.panelWine = wx.Panel(self, -1)
        self.AddPage(self.panelWine, nom)
        # Les polices
        self.txtGeneral = wx.StaticText(self.panelWine, -1, "Wine", (10, 10), wx.DefaultSize)
        self.txtGeneral.SetFont(self.fontTitle)

        self.winecfg_image = wx.Image(Variables.playonlinux_env + "/resources/images/configure/wine-winecfg.png",
                                      wx.BITMAP_TYPE_ANY).ConvertToBitmap()
        self.winecfg = wx.BitmapButton(self.panelWine, id=100, bitmap=self.winecfg_image, pos=(30, 50),
                                       size=(self.winecfg_image.GetWidth() + 5, self.winecfg_image.GetHeight() + 5))
        self.winecfg_texte = wx.StaticText(self.panelWine, -1, _("Configure Wine"), (32, 156), style=wx.ALIGN_CENTER)
        self.winecfg_texte.Wrap(110)
        self.winecfg_texte.SetPosition((self.winecfg_texte.GetPosition()[0] + (
                    105 - self.winecfg_texte.GetSize()[0]) / 2, self.winecfg_texte.GetPosition()[1]))

        self.winecfg_texte.SetFont(self.caption_font)

        self.regedit_image = wx.Image(Variables.playonlinux_env + "/resources/images/configure/registry.png",
                                      wx.BITMAP_TYPE_ANY).ConvertToBitmap()
        self.regedit = wx.BitmapButton(self.panelWine, id=101, bitmap=self.regedit_image, pos=(166, 50),
                                       size=(self.regedit_image.GetWidth() + 5, self.regedit_image.GetHeight() + 5))
        self.regedit_texte = wx.StaticText(self.panelWine, -1, _("Registry Editor"), (168, 156), style=wx.ALIGN_CENTER)
        self.regedit_texte.Wrap(110)
        self.regedit_texte.SetPosition((self.regedit_texte.GetPosition()[0] + (
                    105 - self.regedit_texte.GetSize()[0]) / 2, self.regedit_texte.GetPosition()[1]))

        self.regedit_texte.SetFont(self.caption_font)

        self.wineboot_image = wx.Image(Variables.playonlinux_env + "/resources/images/configure/reboot.png",
                                       wx.BITMAP_TYPE_ANY).ConvertToBitmap()
        self.wineboot = wx.BitmapButton(self.panelWine, id=102, bitmap=self.wineboot_image, pos=(302, 50),
                                        size=(self.wineboot_image.GetWidth() + 5, self.wineboot_image.GetHeight() + 5))
        self.wineboot_texte = wx.StaticText(self.panelWine, -1, _("Windows reboot"), (304, 156), style=wx.ALIGN_CENTER)
        self.wineboot_texte.Wrap(110)
        self.wineboot_texte.SetPosition((self.wineboot_texte.GetPosition()[0] + (
                    105 - self.wineboot_texte.GetSize()[0]) / 2, self.wineboot_texte.GetPosition()[1]))
        self.wineboot_texte.SetFont(self.caption_font)

        self.updatep_image = wx.Image(Variables.playonlinux_env + "/resources/images/configure/update.png",
                                      wx.BITMAP_TYPE_ANY).ConvertToBitmap()
        self.updatep = wx.BitmapButton(self.panelWine, id=107, bitmap=self.updatep_image, pos=(438, 50),
                                       size=(self.wineboot_image.GetWidth() + 5, self.updatep_image.GetHeight() + 5))
        self.updatep_texte = wx.StaticText(self.panelWine, -1, _("Repair virtual drive"), (440, 156),
                                           style=wx.ALIGN_CENTER)
        self.updatep_texte.Wrap(110)
        self.updatep_texte.SetPosition((self.updatep_texte.GetPosition()[0] + (
                    105 - self.wineboot_texte.GetSize()[0]) / 2, self.updatep_texte.GetPosition()[1]))
        self.updatep_texte.SetFont(self.caption_font)

        self.cmd_image = wx.Image(Variables.playonlinux_env + "/resources/images/configure/console.png",
                                  wx.BITMAP_TYPE_ANY).ConvertToBitmap()
        self.cmd = wx.BitmapButton(self.panelWine, id=103, bitmap=self.cmd_image, pos=(30, 196),
                                   size=(self.cmd_image.GetWidth() + 5, self.cmd_image.GetHeight() + 5))
        self.cmd_texte = wx.StaticText(self.panelWine, -1, _("Command prompt"), (32, 302), style=wx.ALIGN_CENTER)
        self.cmd_texte.Wrap(110)
        self.cmd_texte.SetPosition((self.cmd_texte.GetPosition()[0] + (105 - self.cmd_texte.GetSize()[0]) / 2,
                                    self.cmd_texte.GetPosition()[1]))
        self.cmd_texte.SetFont(self.caption_font)

        self.taskmgr_image = wx.Image(Variables.playonlinux_env + "/resources/images/configure/monitor.png",
                                      wx.BITMAP_TYPE_ANY).ConvertToBitmap()
        self.taskmgr = wx.BitmapButton(self.panelWine, id=104, bitmap=self.taskmgr_image, pos=(166, 196),
                                       size=(self.taskmgr_image.GetWidth() + 5, self.taskmgr_image.GetHeight() + 5))
        self.taskmgr_texte = wx.StaticText(self.panelWine, -1, _("Task manager"), (168, 302), style=wx.ALIGN_CENTER)
        self.taskmgr_texte.Wrap(110)
        self.taskmgr_texte.SetPosition((self.taskmgr_texte.GetPosition()[0] + (
                    105 - self.taskmgr_texte.GetSize()[0]) / 2, self.taskmgr_texte.GetPosition()[1]))

        self.taskmgr_texte.SetFont(self.caption_font)

        self.killall_image = wx.Image(Variables.playonlinux_env + "/resources/images/configure/stop.png",
                                      wx.BITMAP_TYPE_ANY).ConvertToBitmap()
        self.killall = wx.BitmapButton(self.panelWine, id=105, bitmap=self.killall_image, pos=(302, 196),
                                       size=(self.killall_image.GetWidth() + 5, self.killall_image.GetHeight() + 5))
        self.killall_texte = wx.StaticText(self.panelWine, -1, _("Kill processes"), (304, 302), style=wx.ALIGN_CENTER)
        self.killall_texte.Wrap(110)
        self.killall_texte.SetPosition((self.killall_texte.GetPosition()[0] + (
                    105 - self.killall_texte.GetSize()[0]) / 2, self.killall_texte.GetPosition()[1]))
        self.killall_texte.SetFont(self.caption_font)

        self.control_image = wx.Image(Variables.playonlinux_env + "/resources/images/configure/control.png",
                                      wx.BITMAP_TYPE_ANY).ConvertToBitmap()
        self.control = wx.BitmapButton(self.panelWine, id=108, bitmap=self.control_image, pos=(438, 196),
                                       size=(self.control_image.GetWidth() + 5, self.control_image.GetHeight() + 5))
        self.control_texte = wx.StaticText(self.panelWine, -1, _("Control panel"), (440, 302), style=wx.ALIGN_CENTER)
        self.control_texte.Wrap(110)
        self.control_texte.SetPosition((self.control_texte.GetPosition()[0] + (
                    105 - self.control_texte.GetSize()[0]) / 2, self.control_texte.GetPosition()[1]))

        self.control_texte.SetFont(self.caption_font)

        wx.EVT_BUTTON(self, 100, self.evt_winecfg)
        wx.EVT_BUTTON(self, 101, self.evt_regedit)
        wx.EVT_BUTTON(self, 102, self.evt_wineboot)
        wx.EVT_BUTTON(self, 103, self.evt_cmd)
        wx.EVT_BUTTON(self, 104, self.evt_taskmgr)
        wx.EVT_BUTTON(self, 105, self.evt_killall)
        wx.EVT_BUTTON(self, 106, self.evt_config)
        wx.EVT_BUTTON(self, 107, self.evt_rep)
        wx.EVT_BUTTON(self, 108, self.evt_control)

    def Packages(self, nom):
        self.panelPackages = wx.Panel(self, -1)
        self.txtPackages = wx.StaticText(self.panelPackages, -1, _(nom), (10, 10), wx.DefaultSize)
        self.txtPackages.SetFont(self.fontTitle)

        self.imagePackages = wx.ImageList(22, 22)

        self.desPackags = wx.StaticText(self.panelPackages, -1, _(
            "Be careful! Installing one of these components can break your virtual drive."), (10, 40), wx.DefaultSize)

        self.Menu = wx.TreeCtrl(self.panelPackages, 99, pos=(15, 75), size=(530, 260),
                                style=wx.TR_HIDE_ROOT | wx.TR_FULL_ROW_HIGHLIGHT | Variables.widget_borders)
        self.Menu.SetSpacing(0);
        self.Menu.SetImageList(self.imagePackages)
        self.imagePackages.RemoveAll()

        self.rootPackages = self.Menu.AddRoot("")
        self.i = 0

        for app in self.packageList.getParsedList():
            self.icon_look_for = Variables.playonlinux_rep + "/configurations/icones/" + self.packageList.getPackageFromName(
                app)
            if (os.path.exists(self.icon_look_for)):
                try:
                    self.imagePackages.Add(wx.Bitmap(self.icon_look_for))
                except:
                    pass
            else:
                self.imagePackages.Add(wx.Bitmap(Variables.playonlinux_env + "/etc/playonlinux22.png"))
            self.Menu.AppendItem(self.rootPackages, app, self.i)
            self.i = self.i + 1

        self.PackageButton = wx.Button(self.panelPackages, 98, _("Install"), pos=(20 + 530 - 150, 345), size=(150, 30))
        self.PackageButton.Disable()

        wx.EVT_TREE_SEL_CHANGED(self, 99, self.package_selected)
        wx.EVT_TREE_ITEM_ACTIVATED(self, 99, self.install_package)
        wx.EVT_BUTTON(self, 98, self.install_package)

        self.AddPage(self.panelPackages, nom)

    def package_selected(self, event):
        selectedPackageName = self.Menu.GetItemText(self.Menu.GetSelection())
        if selectedPackageName:
            self.PackageButton.Enable()
        else:
            self.PackageButton.Disable()

    def change_Direct3D_settings(self, param):
        if (self.s_isPrefix == False):
            subprocess.Popen(
                ["bash", Variables.playonlinux_env + "/bash/POL_Command", self.s_title.encode('utf-8', 'replace'),
                 "POL_Wine_Direct3D", param, self.display_elements[param].GetValue().encode('utf-8', 'replace')])
        else:
            subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/POL_Command", "--prefix",
                              self.s_prefix.encode('utf-8', 'replace'), "POL_Wine_Direct3D", param,
                              self.display_elements[param].GetValue().encode('utf-8', 'replace')])

    def change_DirectInput_settings(self, param):
        if (self.s_isPrefix == False):
            subprocess.Popen(
                ["bash", Variables.playonlinux_env + "/bash/POL_Command", self.s_title.encode('utf-8', 'replace'),
                 "POL_Wine_DirectInput", param, self.display_elements[param].GetValue().encode('utf-8', 'replace')])
        else:
            subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/POL_Command", "--prefix",
                              self.s_prefix.encode('utf-8', 'replace'), "POL_Wine_DirectInput", param,
                              self.display_elements[param].GetValue().encode('utf-8', 'replace')])

    def get_current_settings(self, param):
        self.display_elements[param].SetValue(self.settings[param])

    def UpdateVersions(self, arch):
        elements = playonlinux.Get_versions(arch)
        self.general_elements["wineversion"].Clear()
        if (arch == playonlinux.GetSettings("WINE_SYSTEM_ARCH") or (
                arch == "x86" and playonlinux.GetSettings("WINE_SYSTEM_ARCH") != "amd64")):
            self.general_elements["wineversion"].Append("System")
        self.general_elements["wineversion"].AppendItems(elements)
        version = playonlinux.GetSettings('VERSION', self.s_prefix)
        if (version == ''):
            self.general_elements["wineversion"].SetValue('System')
        else:
            self.general_elements["wineversion"].SetValue(version)

    def UpdateValues(self, selection):
        # print "Test"
        if (self.s_isPrefix == False):
            self.ChangeTitle(selection)
            # self.general_elements["wineversion"].SetValue(wine_versions.GetWineVersion(selection))
            # self.general_elements["wineversion"].Show()
            self.general_elements["wineprefix"].Show()
            self.general_elements["arguments"].Show()
            self.general_elements["arguments_text"].Show()

            # self.general_elements["name"].Show()
            # self.general_elements["wineversion_text"].Show()
            self.general_elements["wineprefix_text"].Show()
            self.general_elements["name"].SetEditable(True)

            # self.general_elements["name_text"].Show()
            self.general_elements["wineprefix"].SetValue(playonlinux.getPrefix(self.s_title))
            self.general_elements["arguments"].SetValue(playonlinux.getArgs(self.s_title))

            self.display_elements["folder_button"].SetLabel(_("Open program's directory"))
            if not playonlinux.GetSettings("OPEN_IN", self.s_prefix):
                self.display_elements["open_in"].SetValue("xdg-open")
            else:
                self.display_elements["open_in"].SetValue(playonlinux.GetSettings("OPEN_IN", self.s_prefix))

            if (os.path.exists(Variables.playonlinux_rep + "configurations/configurators/" + self.s_title)):
                self.configurator_title.Show()
                self.configurator_button.Show()
            else:
                self.configurator_title.Hide()
                self.configurator_button.Hide()
            self.configurator_title.SetLabel(
                "{0} specific configuration".format(self.s_title.encode('utf-8', 'replace')))
            self.display_elements["pre_run_panel"].Show()
            self.display_elements["pre_run_text"].Show()
        else:
            self.s_prefix = selection
            self.s_title = selection
            # self.general_elements["wineversion"].Hide()
            self.general_elements["wineprefix"].Hide()
            # self.general_elements["name"].Hide()
            self.general_elements["name"].SetEditable(False)
            self.general_elements["name"].SetValue(self.s_prefix)
            self.general_elements["arguments"].Hide()
            self.general_elements["arguments_text"].Hide()
            # self.general_elements["wineversion_text"].Hide()
            self.general_elements["wineprefix_text"].Hide()
            # self.general_elements["name_text"].Hide()
            self.display_elements["folder_button"].SetLabel(_("Open virtual drive's directory"))
            self.configurator_title.Hide()
            self.configurator_button.Hide()
            self.display_elements["pre_run_panel"].Hide()
            self.display_elements["pre_run_text"].Hide()

        self.Refresh()
        self.elements = ["UseGLSL", "DirectDrawRenderer", "VideoMemorySize", "OffscreenRenderingMode",
                         "RenderTargetModeLock", "Multisampling", "StrictDrawOrdering", "MouseWarpOverride"]
        self.settings = wine.LoadRegValues(self.s_prefix, self.elements)
        # print self.settings
        self.get_current_settings("UseGLSL")
        self.get_current_settings("DirectDrawRenderer")
        self.get_current_settings("VideoMemorySize")
        self.get_current_settings("OffscreenRenderingMode")
        self.get_current_settings("RenderTargetModeLock")
        self.get_current_settings("Multisampling")
        self.get_current_settings("StrictDrawOrdering")
        self.get_current_settings("MouseWarpOverride")

        self.arch = playonlinux.GetSettings('ARCH', self.s_prefix)
        if (self.arch == ""):
            self.arch = "x86"

        self.UpdateVersions(self.arch)
        self.general_elements["winedebug"].SetValue(playonlinux.GetSettings("WINEDEBUG", self.s_prefix))
        try:
            self.display_elements["pre_run"].SetValue(
                open(os.environ["POL_USER_ROOT"] + "/configurations/pre_shortcut/" + self.s_title, 'r').read())
        except:
            self.display_elements["pre_run"].SetValue("")

    def change_settings(self, event):
        param = event.GetId()
        if (param == 301):
            self.change_Direct3D_settings("UseGLSL")
        if (param == 302):
            self.change_Direct3D_settings("DirectDrawRenderer")
        if (param == 303):
            self.change_Direct3D_settings("VideoMemorySize")
        if (param == 304):
            self.change_Direct3D_settings("OffscreenRenderingMode")
        if (param == 305):
            self.change_Direct3D_settings("RenderTargetModeLock")
        if (param == 306):
            self.change_Direct3D_settings("Multisampling")
        if (param == 307):
            self.change_Direct3D_settings("StrictDrawOrdering")
        if (param == 401):
            self.change_DirectInput_settings("MouseWarpOverride")

    def misc_button(self, event):
        param = event.GetId()
        if (param == 402):
            if (self.s_isPrefix == False):
                playonlinux.open_folder(self.s_title,
                                        self.display_elements["open_in"].GetValue().encode("utf-8", "replace"))
            else:
                playonlinux.open_folder_prefix(self.s_prefix)
        if (param == 404):
            if (self.s_isPrefix == False):
                subprocess.Popen(
                    ["bash", Variables.playonlinux_env + "/bash/POL_Command", self.s_title.encode('utf-8', 'replace'),
                     "POL_OpenShell", self.s_title.encode('utf-8', 'replace')])
            else:
                subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/POL_Command", "--prefix",
                                  self.s_prefix.encode('utf-8', 'replace'), "POL_OpenShell"])

        if (param == 405):
            self.FileDialog = wx.FileDialog(self)
            self.FileDialog.SetDirectory("~")
            self.supported_files = "All|*.exe;*.EXE;*.msi;*.MSI\
            \|Windows executable (*.exe)|*.exe;*.EXE\
            \|Windows install file (*.msi)|*.msi;*MSI"
            self.FileDialog.SetWildcard(self.supported_files)
            self.FileDialog.ShowModal()
            if (self.FileDialog.GetPath() != ""):
                filename = self.FileDialog.GetPath().encode("utf-8", "replace")
                dirname = os.path.dirname(filename)
                if (self.s_isPrefix == True):
                    subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/POL_Command", "--prefix",
                                      self.s_prefix.encode('utf-8', 'replace'), "POL_AutoWine", filename], cwd=dirname)
                else:
                    subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/POL_Command",
                                      self.s_title.encode('utf-8', 'replace'), "POL_AutoWine", filename], cwd=dirname)

        if (param == 201):
            if (self.s_isPrefix == False):
                subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/POL_Command", "--init",
                                  self.s_title.encode('utf-8', 'replace'), "POL_SetupWindow_shortcut_creator"])
            else:
                subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/POL_Command", "--init", "--prefix",
                                  self.s_prefix.encode('utf-8', 'replace'), "POL_SetupWindow_shortcut_creator"])

    def AddDisplayElement(self, title, shortname, elements, wine, num):
        elements.insert(0, "Default")
        wine.insert(0, "default")
        elemsize = (230, 25)
        self.display_elements[shortname + "_text"] = wx.StaticText(self.panelDisplay, -1, title,
                                                                   pos=(15, 19 + num * 40))

        self.display_elements[shortname] = wx.ComboBox(self.panelDisplay, 300 + num, style=wx.CB_READONLY,
                                                       pos=(300, 17 + num * 40), size=elemsize)
        self.display_elements[shortname].AppendItems(wine)
        self.display_elements[shortname].SetValue(wine[0])
        wx.EVT_COMBOBOX(self, 300 + num, self.change_settings)

    def AddMiscElement(self, title, shortname, elements, wine, num):
        elements.insert(0, "Default")
        wine.insert(0, "default")
        elemsize = (230, 25)
        self.display_elements[shortname + "_text"] = wx.StaticText(self.panelMisc, -1, title, pos=(15, 19 + num * 40))

        self.display_elements[shortname] = wx.ComboBox(self.panelMisc, 400 + num, style=wx.CB_READONLY,
                                                       pos=(300, 17 + num * 40), size=elemsize)
        self.display_elements[shortname].AppendItems(wine)
        self.display_elements[shortname].SetValue(wine[0])
        wx.EVT_COMBOBOX(self, 400 + num, self.change_settings)

    def AddMiscButton(self, title, shortname, num):
        self.display_elements[shortname + "_button"] = wx.Button(self.panelMisc, 400 + num, "", pos=(15, 19 + num * 40),
                                                                 size=(500, 30))
        self.display_elements[shortname + "_button"].SetLabel(title)

        wx.EVT_BUTTON(self, 400 + num, self.misc_button)

    def AddMiscChamp(self, title, shortname, value, num):
        self.display_elements[shortname + "_text"] = wx.StaticText(self.panelMisc, -1, title, pos=(15, 24 + num * 40))
        self.display_elements[shortname] = wx.TextCtrl(self.panelMisc, 400 + num, value, pos=(245, 19 + num * 40),
                                                       size=(270, 25))
        wx.EVT_TEXT(self, 400 + num, self.set_open_in)

    def set_open_in(self, event):
        new_open_in = self.display_elements["open_in"].GetValue()
        playonlinux.SetSettings('OPEN_IN', new_open_in, self.s_prefix)

    def AddMiscLongText(self, title, shortname, num):
        self.display_elements[shortname + "_text"] = wx.StaticText(self.panelMisc, -1, title, pos=(15, 19 + num * 40))
        self.display_elements[shortname + "_panel"] = wx.Panel(self.panelMisc, -1, size=wx.Size(450, 70),
                                                               pos=(20, 44 + num * 40))

        try:
            content = open(os.environ["POL_USER_ROOT"] + "/configurations/pre_shortcut/" + self.s_title, 'r').read()
        except:
            content = ""

        self.display_elements[shortname] = wx.TextCtrl(self.display_elements[shortname + "_panel"], 400 + num, content,
                                                       size=wx.Size(448, 68), pos=(2, 2),
                                                       style=Variables.widget_borders | wx.TE_MULTILINE)
        wx.EVT_TEXT(self, 400 + num, self.edit_shortcut)

    def edit_shortcut(self, event):
        content = self.display_elements["pre_run"].GetValue().encode("utf-8", "replace")
        open(os.environ["POL_USER_ROOT"] + "/configurations/pre_shortcut/" + self.s_title, 'w').write(content)

    def AddGeneralButton(self, title, shortname, num):
        self.general_elements[shortname + "_button"] = wx.Button(self.panelGeneral, 200 + num, "",
                                                                 pos=(15, 9 + num * 40), size=(520, 30))
        self.general_elements[shortname + "_button"].SetLabel(title)

        wx.EVT_BUTTON(self, 200 + num, self.misc_button)

    def Display(self, nom):
        self.display_elements = {}
        self.panelDisplay = wx.Panel(self, -1)

        self.txtDisplay = wx.StaticText(self.panelDisplay, -1, _(nom), (10, 10), wx.DefaultSize)
        self.txtDisplay.SetFont(self.fontTitle)

        self.AddPage(self.panelDisplay, nom)
        self.AddDisplayElement(_("GLSL Support"), "UseGLSL", ["Enabled", "Disabled"], ["enabled", "disabled"], 1)
        self.AddDisplayElement(_("Direct Draw Renderer"), "DirectDrawRenderer", ["GDI", "OpenGL"], ["gdi", "opengl"], 2)
        self.AddDisplayElement(_("Video memory size"), "VideoMemorySize",
                               ["32", "64", "128", "256", "384", "512", "768", "1024", "2048", "3072", "4096"],
                               ["32", "64", "128", "256", "384", "512", "768", "1024", "2048", "3072", "4096"], 3)
        self.AddDisplayElement(_("Offscreen rendering mode"), "OffscreenRenderingMode",
                               ["fbo", "backbuffer", "pbuffer"], ["fbo", "backbuffer", "pbuffer"], 4)
        self.AddDisplayElement(_("Render target mode lock"), "RenderTargetModeLock",
                               ["disabeld", "readdraw", "readtex"], ["disabled", "readdraw", "readtex"], 5)
        self.AddDisplayElement(_("Multisampling"), "Multisampling", ["Enabled", "Disabled"], ["enabled", "disabled"], 6)
        self.AddDisplayElement(_("Strict Draw Ordering"), "StrictDrawOrdering", ["enabled", "disabled"],
                               ["enabled", "disabled"], 7)

    def Miscellaneous(self, nom):
        self.misc_elements = {}
        self.panelMisc = wx.Panel(self, -1)

        self.txtMisc = wx.StaticText(self.panelMisc, -1, _(nom), (10, 10), wx.DefaultSize)
        self.txtMisc.SetFont(self.fontTitle)

        self.AddMiscElement(_("Mouse warp override"), "MouseWarpOverride", ["Enabled", "Disabled", "Force"],
                            ["enable", "disable", "force"], 1)
        self.AddMiscButton("", "folder", 2)
        self.AddMiscChamp(_("Open directory using command"), "open_in", "", 3)
        self.AddMiscButton(_("Open a shell"), "shell", 4)
        self.AddMiscButton(_("Run a .exe file in this virtual drive"), "exerun", 5)
        self.AddMiscLongText(_("Command to exec before running the program"), "pre_run", 6)

        self.AddPage(self.panelMisc, nom)

    def assign(self, event):
        old_version = playonlinux.GetSettings('VERSION', self.s_prefix)
        if old_version == '':
            old_version = 'System'
        version = self.general_elements["wineversion"].GetValue()
        if version != old_version:
            if (wx.YES == wx.MessageBox(_(
                    "Warning:\n\nAny program still running in this virtual drive will be terminated before Wine version is changed.\nAre you sure you want to continue?"),
                                        os.environ["APPLICATION_TITLE"], style=wx.YES_NO | wx.ICON_QUESTION)):
                self.evt_killall(event)
                if version != 'System':
                    playonlinux.SetSettings('VERSION', version, self.s_prefix)
                else:
                    playonlinux.DeleteSettings('VERSION', self.s_prefix)
            else:
                version = old_version
                self.general_elements["wineversion"].SetValue(old_version)

    def assignPrefix(self, event):
        if (wx.YES == wx.MessageBox(_(
                "Be careful!\nIf you change " + self.s_title + "'s virtual drive, you are likely to break it.\nDo this only if you know what you are doing.\n\nAre you sure you want to continue?"),
                                    os.environ["APPLICATION_TITLE"], style=wx.YES_NO | wx.ICON_QUESTION)):
            drive = self.general_elements["wineprefix"].GetValue()
            playonlinux.SetWinePrefix(self.s_title, drive)
        else:
            self.general_elements["wineprefix"].SetValue(self.s_prefix)

    def ReleaseTyping(self, event):
        self.typing = False

    def setargs(self, event):
        new_args = self.general_elements["arguments"].GetValue()
        playonlinux.writeArgs(self.s_title, new_args)

    def setwinedebug(self, event):
        new_winedebug = self.general_elements["winedebug"].GetValue()
        playonlinux.SetSettings('WINEDEBUG', new_winedebug, self.s_prefix)

    def setname(self, event):
        new_name = self.general_elements["name"].GetValue()
        if (self.changing_selection == False):
            self.typing = True
        else:
            self.changing_selection = False

        if (not os.path.exists(Variables.playonlinux_rep + "shortcuts/" + new_name)):
            try:
                os.rename(Variables.playonlinux_rep + "icones/32/" + self.s_title,
                          Variables.playonlinux_rep + "icones/32/" + new_name)
            except:
                pass

            try:
                os.rename(Variables.playonlinux_rep + "icones/full_size/" + self.s_title,
                          Variables.playonlinux_rep + "icones/full_size/" + new_name)
            except:
                pass

            try:
                os.rename(Variables.playonlinux_rep + "configurations/configurators/" + self.s_title,
                          Variables.playonlinux_rep + "configurations/configurators/" + new_name)
            except:
                pass

            try:
                os.rename(Variables.playonlinux_rep + "shortcuts/" + self.s_title,
                          Variables.playonlinux_rep + "shortcuts/" + new_name)
                self.s_title = new_name
                self.s_prefix = playonlinux.getPrefix(self.s_title)
            except:
                pass

            # if(self.changing == False):
            #       self.Parent.Parent.list_software()
            # else:
            #       self.changing = False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2008 Pâris Quentin
# Copyright (C) 2007-2057 PlayOnLinux team

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
encoding = 'utf-8'

import os
import shlex
import signal
import string
import subprocess
import sys
import time
import urllib
import webbrowser

try:
    os.environ["POL_OS"]
except:
    print "ERROR ! Please define POL_OS environment var first."
    os._exit(1)

if (os.environ["POL_OS"] != "Mac"):
    import wxversion

    wxversion.ensureMinimal('2.8')

import wx, wx.aui
import wx.lib.hyperlink
import lib.lng as lng
import lib.playonlinux as playonlinux, lib.Variables as Variables
import options, threading, debug
from configurewindow import ConfigureWindow
from wine_versions import WineVersionsWindow as wver
from setupwindow import gui_server
from install.InstallWindow import InstallWindow


# This thread manage updates
class POLWeb(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)
        self.sendToStatusBarStr = ""
        self.sendAlertStr = None
        self.Gauge = False
        self.WebVersion = ""
        self.Show = False
        self.perc = -1
        self.updating = True

    def sendToStatusBar(self, message, gauge):
        self.sendToStatusBarStr = message
        self.Gauge = gauge
        self.Show = True

    def sendPercentage(self, n):
        self.perc = n

    def sendAlert(self, message):
        self.sendAlertStr = message

    def LastVersion(self):
        if (os.environ["POL_OS"] == "Mac"):
            fichier_online = "version_mac"
        elif (os.environ["POL_OS"] == "FreeBSD"):
            fichier_online = "version_freebsd"
        else:
            fichier_online = "version2"
        return os.popen(
            os.environ["POL_WGET"] + ' "' + os.environ["SITE"] + '/' + fichier_online + '.php?v=' + os.environ[
                "VERSION"] + '" -T 30 -O-', 'r').read()

    def real_check(self):
        self.WebVersion = self.LastVersion()

        if (self.WebVersion == ""):
            self.sendToStatusBar(
                _('{0} website is unavailable. Please check your connection').format(os.environ["APPLICATION_TITLE"]),
                False)
        else:
            self.sendToStatusBar(_("Refreshing {0}").format(os.environ["APPLICATION_TITLE"]), True)
            self.sendPercentage(0)
            self.updating = True
            exe = ['bash', Variables.playonlinux_env + "/bash/pol_update_list"]

            p = subprocess.Popen(exe, stdout=subprocess.PIPE, bufsize=1,
                                 preexec_fn=lambda: os.setpgid(os.getpid(), os.getpid()))

            for line in iter(p.stdout.readline, ''):
                try:
                    self.sendPercentage(int(line))
                except ValueError:
                    pass

            self.updating = False
            if (playonlinux.VersionLower(os.environ["VERSION"], self.WebVersion)):
                self.sendToStatusBar(_('An updated version of {0} is available').format(
                    os.environ["APPLICATION_TITLE"]) + " (" + self.WebVersion + ")", False)
                if (os.environ["DEBIAN_PACKAGE"] == "FALSE"):
                    self.sendAlert(_('An updated version of {0} is available').format(
                        os.environ["APPLICATION_TITLE"]) + " (" + self.WebVersion + ")")
                os.environ["POL_UPTODATE"] = "FALSE"
            else:
                self.Show = False
                self.perc = -1
                os.environ["POL_UPTODATE"] = "TRUE"

        self.wantcheck = False

    def check(self):
        self.wantcheck = True

    def run(self):
        self.check()
        while (1):
            if (self.wantcheck == True):
                self.real_check()
            time.sleep(1)


class PanelManager(wx.aui.AuiManager):

    def __init__(self, frame):
        wx.aui.AuiManager.__init__(self, frame)
        self.startPerspective = self.SavePerspective()

    def _getPerspectiveName(self):
        name = self.SavePerspective().split("=")
        name = name[1].split(";")
        name = name[0]
        return name

    def getPerspective(self):
        return self.SavePerspective().replace(self._getPerspectiveName(), "PERSPECTIVE_NAME")

    def savePosition(self):
        playonlinux.SetSettings("PANEL_PERSPECTIVE", self.getPerspective())

    def restorePosition(self):
        self.startPerspective = self.SavePerspective()

        self.Update()

        try:
            setting = playonlinux.GetSettings("PANEL_PERSPECTIVE")
            if (setting.count("Actions") < 2 or setting.count("dock_size") < 2 or not "PERSPECTIVE_NAME" in setting):
                self.LoadPerspective(self.startPerspective)
            else:
                setting = setting.replace("PERSPECTIVE_NAME", self._getPerspectiveName())
                if (setting != ""):
                    self.LoadPerspective(setting)

        except wx._core.PyAssertionError:
            self.LoadPerspective(self.startPerspective)

        self.Update()

    def AddPane(self, data, settings):
        wx.aui.AuiManager.AddPane(self, data, settings)

    def Destroy(self):
        self.savePosition()


class MainWindow(wx.Frame):
    def __init__(self, parent, id, title):

        wx.Frame.__init__(self, parent, 1000, title, size=(515, 450))
        self.SetMinSize((400, 400))
        self.SetIcon(wx.Icon(Variables.playonlinux_env + "/etc/playonlinux.png", wx.BITMAP_TYPE_ANY))

        self.windowList = {}
        self.registeredPid = []
        self.windowOpened = 0

        # Manage updater
        self.updater = POLWeb()
        self.updater.start()

        # These lists contain the dock links and images
        self.menuElem = {}
        self.menuImage = {}

        # Catch CTRL+C
        signal.signal(signal.SIGINT, self.ForceClose)

        # Window size
        try:
            self.windowWidth = int(playonlinux.GetSettings("MAINWINDOW_WIDTH"))
            self.windowHeight = int(playonlinux.GetSettings("MAINWINDOW_HEIGHT"))
            self.SetSize((self.windowWidth, self.windowHeight))
        except:
            self.windowWidth = 500
            self.windowHeight = 450

        # Window position
        try:
            self.windowx = int(playonlinux.GetSettings("MAINWINDOW_X"))
            self.windowy = int(playonlinux.GetSettings("MAINWINDOW_Y"))
            self.screen_width = wx.Display().GetGeometry()[2]
            self.screen_height = wx.Display().GetGeometry()[3]

            if (self.screen_width >= self.windowx and self.screen_height >= self.windowy):
                self.SetPosition((self.windowx, self.windowy))
            else:
                self.Center(wx.BOTH)
        except:
            self.Center(wx.BOTH)

        try:
            self.iconSize = int(playonlinux.GetSettings("ICON_SIZE"))
        except:
            self.iconSize = 32

        self.images = wx.ImageList(self.iconSize, self.iconSize)
        self.imagesEmpty = wx.ImageList(1, 1)

        self.sendAlertStr = None

        ## Fonts
        if (os.environ["POL_OS"] == "Mac"):
            self.fontTitre = wx.Font(14, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD, False, "",
                                     wx.FONTENCODING_DEFAULT)
            self.fontText = wx.Font(12, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL, False, "",
                                    wx.FONTENCODING_DEFAULT)
        else:
            self.fontTitre = wx.Font(10, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD, False, "",
                                     wx.FONTENCODING_DEFAULT)
            self.fontText = wx.Font(8, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL, False, "",
                                    wx.FONTENCODING_DEFAULT)

        ## List game
        self.list_game = wx.TreeCtrl(self, 105, style=wx.TR_HIDE_ROOT | wx.TR_FULL_ROW_HIGHLIGHT)
        self.list_game.SetSpacing(0)
        self.list_game.SetIndent(5)
        self.list_game.SetImageList(self.images)
        self.menu_gauche = wx.Panel(self, -1)

        self._mgr = PanelManager(self)
        self._mgr.AddPane(self.list_game, wx.CENTER)
        self._mgr.AddPane(self.menu_gauche,
                          wx.aui.AuiPaneInfo().Name('Actions').Caption('Actions').Left().BestSize((200, 400)).Floatable(
                              True).CloseButton(False).TopDockable(False).BottomDockable(False))

        self.filemenu = wx.Menu()
        ### On MacOS X, preference is always on the main menu
        if (os.environ["POL_OS"] == "Mac"):
            prefItem = self.filemenu.Append(wx.ID_PREFERENCES, text="&Preferences")
            self.Bind(wx.EVT_MENU, self.Options, prefItem)

        ### File menu
        self.filemenu.Append(wx.ID_OPEN, _("Run"))
        self.filemenu.Append(wx.ID_ADD, _("Install"))
        self.filemenu.Append(wx.ID_DELETE, _("Remove"))
        self.filemenu.AppendSeparator()
        self.filemenu.Append(216, _("Donate"))
        self.filemenu.Append(wx.ID_EXIT, _("Exit"))

        ### Display menu
        self.displaymenu = wx.Menu()
        self.icon16 = self.displaymenu.AppendRadioItem(501, _("Small icons"))
        self.icon24 = self.displaymenu.AppendRadioItem(502, _("Medium icons"))
        self.icon32 = self.displaymenu.AppendRadioItem(503, _("Large icons"))
        self.icon48 = self.displaymenu.AppendRadioItem(504, _("Very large icons"))
        if (self.iconSize == 16):
            self.icon16.Check(True)
        if (self.iconSize == 24):
            self.icon24.Check(True)
        if (self.iconSize == 32):
            self.icon32.Check(True)
        if (self.iconSize == 48):
            self.icon48.Check(True)

        self.expertmenu = wx.Menu()

        self.expertmenu.Append(107, _("Manage Wine versions"))

        if (os.environ["POL_OS"] == "Mac"):
            self.expertmenu.AppendSeparator()
            self.expertmenu.Append(113, _("Read a PC CD-Rom"))

        self.expertmenu.AppendSeparator()

        self.expertmenu.Append(108, _("Run a local script"))
        self.expertmenu.Append(109, _("{0} console").format(os.environ["APPLICATION_TITLE"]))

        self.expertmenu.Append(115, _('Close all {0} software').format(os.environ["APPLICATION_TITLE"]))
        self.expertmenu.AppendSeparator()

        self.expertmenu.Append(110, _("{0} debugger").format(os.environ["APPLICATION_TITLE"]))

        self.optionmenu = wx.Menu()

        self.optionmenu.Append(211, _("Internet"))
        self.optionmenu.Append(212, _("File associations"))
        self.optionmenu.Append(214, _("Plugin manager"))

        self.supportmenu = wx.Menu()
        self.supportmenu.Append(400, _("Supported software"))
        self.supportmenu.Append(401, _("News"))
        self.supportmenu.Append(402, _("Documentation"))
        self.supportmenu.Append(403, _("Forums"))
        self.supportmenu.Append(404, _("Bugs"))
        self.supportmenu.AppendSeparator()
        self.supportmenu.Append(405, _("Twitter"))
        self.supportmenu.Append(406, _("Google+"))
        self.supportmenu.Append(407, _("Facebook"))

        self.help_menu = wx.Menu()
        self.help_menu.Append(wx.ID_ABOUT, _('About {0}').format(os.environ["APPLICATION_TITLE"]))
        self.pluginsmenu = wx.Menu()

        files = os.listdir(Variables.playonlinux_rep + "/plugins")
        files.sort()
        self.plugin_list = []
        self.i = 0
        self.j = 0
        while (self.i < len(files)):
            if (os.path.exists(Variables.playonlinux_rep + "/plugins/" + files[self.i] + "/scripts/menu")):
                if (os.path.exists(Variables.playonlinux_rep + "/plugins/" + files[self.i] + "/enabled")):
                    self.plugin_item = wx.MenuItem(self.expertmenu, 300 + self.j, files[self.i])

                    self.icon_look_for = Variables.playonlinux_rep + "/plugins/" + files[self.i] + "/icon"
                    if (os.path.exists(self.icon_look_for)):
                        self.bitmap = wx.Bitmap(self.icon_look_for)
                    else:
                        self.bitmap = wx.Bitmap(Variables.playonlinux_env + "/etc/playonlinux16.png")

                    self.plugin_item.SetBitmap(self.bitmap)
                    self.pluginsmenu.AppendItem(self.plugin_item)
                    wx.EVT_MENU(self, 300 + self.j, self.run_plugin)
                    self.plugin_list.append(files[self.i])
                    self.j += 1
            self.i += 1

        if (self.j > 0):
            self.pluginsmenu.AppendSeparator()

        self.option_item_p = wx.MenuItem(self.pluginsmenu, 214, _("Plugin manager"))
        self.option_item_p.SetBitmap(wx.Bitmap(Variables.playonlinux_env + "/etc/onglet/package-x-generic.png"))

        self.pluginsmenu.AppendItem(self.option_item_p)

        self.last_string = ""

        self.sb = wx.StatusBar(self, -1)
        self.sb.SetFieldsCount(2)
        self.sb.SetStatusWidths([self.GetSize()[0], -1])
        self.sb.SetStatusText("", 0)

        if (os.environ["POL_OS"] == "Mac"):
            hauteur = 2
        else:
            hauteur = 6
        self.jauge_update = wx.Gauge(self.sb, -1, 100, (self.GetSize()[0] - 100, hauteur), size=(100, 16))
        self.jauge_update.Pulse()
        self.jauge_update.Hide()
        self.SetStatusBar(self.sb)

        # self.helpmenu = wx.MenuItem()
        # self.helpmenu.Append(wx.ID_ABOUT, _("About"))

        self.menubar = wx.MenuBar()
        self.menubar.Append(self.filemenu, _("File"))
        self.menubar.Append(self.displaymenu, _("Display"))
        self.menubar.Append(self.expertmenu, _("Tools"))
        self.menubar.Append(self.optionmenu, _("Settings"))
        self.menubar.Append(self.pluginsmenu, _("Plugins"))
        self.menubar.Append(self.supportmenu, _("Support"))
        self.menubar.Append(self.help_menu, "&Help")

        # self.menubar.Append(self.help_menu, _("About"))

        self.SetMenuBar(self.menubar)
        iconSize = (32, 32)

        self.toolbar = self.CreateToolBar(wx.TB_TEXT)
        self.toolbar.SetToolBitmapSize(iconSize)
        self.searchbox = wx.SearchCtrl(self.toolbar, 124, style=wx.RAISED_BORDER)
        self.playTool = self.toolbar.AddLabelTool(wx.ID_OPEN, _("Run"), wx.Bitmap(
            Variables.playonlinux_env + "/resources/images/toolbar/play.png"))
        self.stopTool = self.toolbar.AddLabelTool(123, _("Close"), wx.Bitmap(
            Variables.playonlinux_env + "/resources/images/toolbar/stop.png"))

        self.toolbar.AddSeparator()
        self.toolbar.AddLabelTool(wx.ID_ADD, _("Install"),
                                  wx.Bitmap(Variables.playonlinux_env + "/resources/images/toolbar/install.png"))
        self.removeTool = self.toolbar_remove = self.toolbar.AddLabelTool(wx.ID_DELETE, _("Remove"), wx.Bitmap(
            Variables.playonlinux_env + "/resources/images/toolbar/delete.png"))
        self.toolbar.AddSeparator()
        self.toolbar.AddLabelTool(121, _("Configure"),
                                  wx.Bitmap(Variables.playonlinux_env + "/resources/images/toolbar/configure.png"))

        try:
            self.toolbar.AddStretchableSpace()
            self.SpaceHack = False
        except:
            #  wxpython 2.8 does not support AddStretchableSpace(). This is a dirty workaround for this.
            self.dirtyHack = wx.StaticText(self.toolbar)
            self.SpaceHack = True
            self.toolbar.AddControl(self.dirtyHack)
            self.UpdateSearchHackSize()

        try:
            self.toolbar.AddControl(self.searchbox, _("Search"))
        except:
            self.toolbar.AddControl(self.searchbox)
            self.searchbox.SetDescriptiveText(_("Search"))

        self.toolbar.Realize()
        self.Reload(self)
        wx.EVT_MENU(self, wx.ID_OPEN, self.Run)
        wx.EVT_MENU(self, 123, self.RKill)

        wx.EVT_MENU(self, wx.ID_ADD, self.InstallMenu)
        wx.EVT_MENU(self, wx.ID_ABOUT, self.About)
        wx.EVT_MENU(self, wx.ID_EXIT, self.ClosePol)
        wx.EVT_MENU(self, wx.ID_DELETE, self.UninstallGame)

        # Display
        wx.EVT_MENU(self, 501, self.iconDisplay)
        wx.EVT_MENU(self, 502, self.iconDisplay)
        wx.EVT_MENU(self, 503, self.iconDisplay)
        wx.EVT_MENU(self, 504, self.iconDisplay)

        # Expert
        wx.EVT_MENU(self, 101, self.Reload)
        wx.EVT_MENU(self, 107, self.WineVersion)
        wx.EVT_MENU(self, 108, self.Executer)
        wx.EVT_MENU(self, 109, self.PolShell)
        wx.EVT_MENU(self, 110, self.BugReport)
        wx.EVT_MENU(self, 112, self.POLOnline)
        wx.EVT_MENU(self, 113, self.PCCd)
        wx.EVT_MENU(self, 115, self.killall)
        wx.EVT_MENU(self, 121, self.Configure)
        wx.EVT_MENU(self, 122, self.Package)
        wx.EVT_TEXT(self, 124, self.Reload)

        # Options
        wx.EVT_MENU(self, 210, self.Options)
        wx.EVT_MENU(self, 211, self.Options)
        wx.EVT_MENU(self, 212, self.Options)
        wx.EVT_MENU(self, 213, self.Options)
        wx.EVT_MENU(self, 214, self.Options)
        wx.EVT_MENU(self, 215, self.Options)

        wx.EVT_MENU(self, 216, self.donate)

        wx.EVT_CLOSE(self, self.ClosePol)
        wx.EVT_TREE_ITEM_ACTIVATED(self, 105, self.Run)
        wx.EVT_TREE_SEL_CHANGED(self, 105, self.Select)

        # Support
        wx.EVT_MENU(self, 400, self.runSupport)
        wx.EVT_MENU(self, 401, self.runSupport)
        wx.EVT_MENU(self, 402, self.runSupport)
        wx.EVT_MENU(self, 403, self.runSupport)
        wx.EVT_MENU(self, 404, self.runSupport)

        # PlayOnLinux main timer
        self.timer = wx.Timer(self, 1)
        self.Bind(wx.EVT_TIMER, self.TimerAction, self.timer)
        self.timer.Start(1000)
        self.Timer_LastShortcutList = None
        self.Timer_LastIconList = None

        # SetupWindow timer. The server is in another thread and GUI must be run from the main thread
        self.SetupWindowTimer = wx.Timer(self, 2)
        self.Bind(wx.EVT_TIMER, self.SetupWindowAction, self.SetupWindowTimer)
        self.SetupWindowTimer_action = None
        self.SetupWindowTimer.Start(100)
        self.SetupWindowTimer_delay = 100

        # Pop-up menu for game list: beginning
        wx.EVT_TREE_ITEM_MENU(self, 105, self.RMBInGameList)
        wx.EVT_MENU(self, 230, self.RWineConfigurator)
        wx.EVT_MENU(self, 231, self.RRegistryEditor)
        wx.EVT_MENU(self, 232, self.GoToAppDir)
        wx.EVT_MENU(self, 233, self.ChangeIcon)
        wx.EVT_MENU(self, 234, self.UninstallGame)
        wx.EVT_MENU(self, 235, self.RKill)
        wx.EVT_MENU(self, 236, self.ReadMe)
        self.Bind(wx.EVT_SIZE, self.ResizeWindow)
        self._mgr.restorePosition()

    def ResizeWindow(self, e):
        self.UpdateGaugePos()
        self.UpdateSearchHackSize()

    def UpdateSearchHackSize(self):
        if (self.SpaceHack == True):
            self.dirtyHack.SetLabel("")
            self.dirtyHack.SetSize((50, 1))

    def UpdateGaugePos(self):
        if (os.environ["POL_OS"] == "Mac"):
            hauteur = 2
        else:
            hauteur = 6
        self.jauge_update.SetPosition((self.GetSize()[0] - 100, hauteur))

    def SetupWindowTimer_SendToGui(self, recvData):
        recvData = recvData.split("\t")
        while (self.SetupWindowTimer_action != None):
            time.sleep(0.1)
        self.SetupWindowTimer_action = recvData

    def SetupWindow_TimerRestart(self, time):
        if (time != self.SetupWindowTimer_delay):
            self.SetupWindowTimer.Stop()
            self.SetupWindowTimer.Start(time)
            self.SetupWindowTimer_delay = time

    def SetupWindowAction(self, event):
        if (self.windowOpened == 0):
            self.SetupWindow_TimerRestart(100)
        else:
            self.SetupWindow_TimerRestart(10)

        if (self.SetupWindowTimer_action != None):
            return gui_server.readAction(self)

    def TimerAction(self, event):
        self.StatusRead()

        # We read shortcut folder to see if it has to be rescanned
        currentShortcuts = os.path.getmtime(Variables.playonlinux_rep + "/shortcuts")
        currentIcons = os.path.getmtime(Variables.playonlinux_rep + "/icones/32")
        if (currentShortcuts != self.Timer_LastShortcutList or currentIcons != self.Timer_LastIconList):
            self.Reload(self)
            self.Timer_LastShortcutList = currentShortcuts
            self.Timer_LastIconList = currentIcons

    def StatusRead(self):
        self.sb.SetStatusText(self.updater.sendToStatusBarStr, 0)
        if (self.updater.Gauge == True):
            perc = self.updater.perc
            if (perc == -1):
                self.jauge_update.Pulse()
            else:
                try:
                    self.installFrame.percentageText.SetLabel(str(perc) + " %")
                except:
                    pass
                self.jauge_update.SetValue(perc)
            self.jauge_update.Show()
        else:
            self.jauge_update.Hide()

        try:
            if (self.updater.updating == True):
                self.sb.Show()
                ## TODO: Refactor
                self.installFrame.setWaitState(True)
            else:
                self.sb.Hide()
                self.installFrame.setWaitState(False)
                self.installFrame.Refresh()
        except wx._core.PyDeadObjectError:
            pass
        except AttributeError:  # FIXME: Install Frame is not opened
            pass

        if (self.updater.sendAlertStr != self.sendAlertStr):
            wx.MessageBox(self.updater.sendAlertStr, os.environ["APPLICATION_TITLE"], wx.OK | wx.CENTER, self)
            self.sendAlertStr = self.updater.sendAlertStr

    def RMBInGameList(self, event):
        self.GameListPopUpMenu = wx.Menu()

        self.ConfigureWine = wx.MenuItem(self.GameListPopUpMenu, 230, _("Configure Wine"))
        self.ConfigureWine.SetBitmap(wx.Bitmap(Variables.playonlinux_env + "/resources/images/menu/run.png"))
        self.GameListPopUpMenu.AppendItem(self.ConfigureWine)

        self.RegistryEditor = wx.MenuItem(self.GameListPopUpMenu, 231, _("Registry Editor"))
        self.RegistryEditor.SetBitmap(wx.Bitmap(Variables.playonlinux_env + "/resources/images/menu/regedit.png"))
        self.GameListPopUpMenu.AppendItem(self.RegistryEditor)

        self.GotoAppDir = wx.MenuItem(self.GameListPopUpMenu, 232, _("Open the application's directory"))
        self.GotoAppDir.SetBitmap(wx.Bitmap(Variables.playonlinux_env + "/resources/images/menu/folder-wine.png"))
        self.GameListPopUpMenu.AppendItem(self.GotoAppDir)

        self.ChangeIcon = wx.MenuItem(self.GameListPopUpMenu, 236, _("Read the manual"))
        self.ChangeIcon.SetBitmap(wx.Bitmap(Variables.playonlinux_env + "/resources/images/menu/manual.png"))
        self.GameListPopUpMenu.AppendItem(self.ChangeIcon)

        self.ChangeIcon = wx.MenuItem(self.GameListPopUpMenu, 233, _("Set the icon"))
        self.ChangeIcon.SetBitmap(wx.Bitmap(Variables.playonlinux_env + "/resources/images/menu/change_icon.png"))
        self.GameListPopUpMenu.AppendItem(self.ChangeIcon)

        self.ChangeIcon = wx.MenuItem(self.GameListPopUpMenu, 234, _("Remove"))
        self.ChangeIcon.SetBitmap(wx.Bitmap(Variables.playonlinux_env + "/resources/images/menu/delete.png"))
        self.GameListPopUpMenu.AppendItem(self.ChangeIcon)

        self.ChangeIcon = wx.MenuItem(self.GameListPopUpMenu, 235, _("Close this application"))
        self.ChangeIcon.SetBitmap(
            wx.Bitmap(Variables.playonlinux_env + "/resources/images/menu/media-playback-stop.png"))
        self.GameListPopUpMenu.AppendItem(self.ChangeIcon)

        self.PopupMenu(self.GameListPopUpMenu, event.GetPoint())

    def RWineConfigurator(self, event):
        self.RConfigure("winecfg")

    def RKill(self, event):
        self.RConfigure("kprocess")

    def ReadMe(self, event):
        game_exec = self.GetSelectedProgram()
        if (os.path.exists(os.environ["POL_USER_ROOT"] + "/configurations/manuals/" + game_exec)):
            playonlinux.POL_Open(os.environ["POL_USER_ROOT"] + "/configurations/manuals/" + game_exec)
        else:
            wx.MessageBox(_("No manual found for {0}").format(game_exec), os.environ["APPLICATION_TITLE"])

    def RRegistryEditor(self, event):
        self.RConfigure("regedit")

    def run_plugin(self, event):
        game_exec = self.GetSelectedProgram()
        plugin = self.plugin_list[event.GetId() - 300]
        try:
            subprocess.Popen(["bash", Variables.playonlinux_rep + "/plugins/" + plugin + "/scripts/menu", game_exec])
        except:
            pass

    def runSupport(self, event):
        urlId = event.GetId() - 400
        urlPrefix = "http://www." + os.environ["POL_DNS"] + "/en"
        if (urlId == 0):
            url = urlPrefix + "/supported_apps.html"
        if (urlId == 1):
            url = urlPrefix + "/news.html"
        if (urlId == 2):
            url = urlPrefix + "/documentation.html"
        if (urlId == 3):
            url = urlPrefix + "/forums.html"
        if (urlId == 4):
            url = urlPrefix + "/bugs.html"

        if (urlId == 5):
            if (os.environ["POL_OS"] == "Mac"):
                url = "https://twitter.com/PlayOnMac"
            else:
                url = "https://twitter.com/PlayOnLinux"

        if (urlId == 6):
            if (os.environ["POL_OS"] == "Mac"):
                url = "http://plus.google.com/u/0/105992880311102680198"
            else:
                url = "https://plus.google.com/+playonlinux"

        if (urlId == 7):
            if (os.environ["POL_OS"] == "Mac"):
                url = "https://www.facebook.com/playonmac"
            else:
                url = "https://www.facebook.com/playonlinux"

        playonlinux.POL_Open(url)

    def iconDisplay(self, event):
        iconEvent = event.GetId()

        if (iconEvent == 501):
            self.iconSize = 16
        if (iconEvent == 502):
            self.iconSize = 24
        if (iconEvent == 503):
            self.iconSize = 32
        if (iconEvent == 504):
            self.iconSize = 48

        playonlinux.SetSettings("ICON_SIZE", str(self.iconSize))
        self.list_game.SetImageList(self.imagesEmpty)
        self.images.Destroy()
        self.images = wx.ImageList(self.iconSize, self.iconSize)
        self.list_game.SetImageList(self.images)

        self.Reload(self)

    def UpdateInstructions(self, event):
        if (os.environ["POL_OS"] == "Mac"):
            webbrowser.open("http://www.playonmac.com/en/download.html")
        else:
            webbrowser.open("http://www.playonlinux.com/en/download.html")

    def UpdateGIT(self, event):
        subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/update_git"])

    def GoToAppDir(self, event):
        self.game_exec = self.GetSelectedProgram()
        if not playonlinux.GetSettings("OPEN_IN", playonlinux.getPrefix(self.game_exec)):
            playonlinux.open_folder(self.game_exec)
        else:
            playonlinux.open_folder(self.game_exec,
                                    playonlinux.GetSettings("OPEN_IN", playonlinux.getPrefix(self.game_exec)))

    def ChangeIcon(self, event):
        self.IconDir = Variables.homedir + "/.local/share/icons/"
        self.SupprotedIconExt = "All|*.xpm;*.XPM;*.png;*.PNG;*.ico;*.ICO;*.jpg;*.JPG;*.jpeg;*.JPEG;*.bmp;*.BMP\
        \|XPM (*.xpm)|*.xpm;*.XPM\
        \|PNG (*.png)|*.png;*.PNG\
        \|ICO (*.ico)|*.ico;*.ICO\
        \|JPG (*.jpg)|*.jpg;*.JPG\
        \|BMP (*.bmp)|*.bmp;*.BMP\
        \|JPEG (*.jpeg)|*.jpeg;*JPEG"
        self.IconDialog = wx.FileDialog(self, "Choose a icon file", self.IconDir, "", self.SupprotedIconExt,
                                        wx.OPEN | wx.FD_PREVIEW)
        if self.IconDialog.ShowModal() == wx.ID_OK:
            self.IconFilename = self.IconDialog.GetFilename()
            self.IconDirname = self.IconDialog.GetDirectory()
            IconFile = os.path.join(self.IconDirname, self.IconFilename)
            self.RConfigure("changeicon", IconFile)
            self.IconDialog.Destroy()
            # Pop-up menu for game list: ending

    def Select(self, event):
        game_exec = self.GetSelectedProgram()
        self.read = open(Variables.playonlinux_rep + "shortcuts/" + game_exec, "r").readlines()
        self.i = 0
        self.wine_present = False
        while (self.i < len(self.read)):
            if ("wine " in self.read[self.i]):
                self.wine_present = True
            self.i += 1

        self.generate_menu(game_exec)
        self.playTool.Enable(True)
        self.stopTool.Enable(True)
        self.removeTool.Enable(True)

    def generate_menu(self, shortcut=None):
        for c in self.menuElem:
            self.menuElem[c].Destroy()

        for c in self.menuImage:
            self.menuImage[c].Destroy()
        try:
            self.menuBitmap.Destroy()
        except:
            pass

        self.menuElem = {}
        self.menuImage = {}

        i = 0
        self.menuGaucheAddTitle("pol_title", os.environ["APPLICATION_TITLE"], i)
        i += 1
        self.menuGaucheAddLink("pol_prgm_install", _("Install a program"), i,
                               Variables.playonlinux_env + "/resources/images/menu/add.png", self.InstallMenu)
        i += 1
        self.menuGaucheAddLink("pol_prgm_settings", _("Settings"), i,
                               Variables.playonlinux_env + "/resources/images/menu/settings.png", self.Options)

        if (os.path.exists(Variables.playonlinux_env + "/.git/")):
            i += 1
            self.menuGaucheAddLink("pol_git", _("Update GIT"), i,
                                   Variables.playonlinux_env + "/resources/images/menu/update_git.png", self.UpdateGIT)
        elif "POL_UPTODATE" in os.environ and os.environ["POL_UPTODATE"] == "FALSE":
            i += 1
            self.menuGaucheAddLink("pol_update", _("Update instructions"), i,
                                   Variables.playonlinux_env + "/resources/images/menu/update_git.png",
                                   self.UpdateInstructions)

        if (shortcut != None):
            i += 2
            self.menuGaucheAddTitle("prgm_title", shortcut, i)
            i += 1
            self.menuGaucheAddLink("pol_prgm_run", _("Run"), i,
                                   Variables.playonlinux_env + "/resources/images/menu/media-playback-start.png",
                                   self.Run)
            i += 1
            self.menuGaucheAddLink("pol_prgm_kill", _("Close"), i,
                                   Variables.playonlinux_env + "/resources/images/menu/media-playback-stop.png",
                                   self.RKill)
            i += 1
            self.menuGaucheAddLink("pol_prgm_rundebug", _("Debug"), i,
                                   Variables.playonlinux_env + "/resources/images/menu/bug.png", self.RunDebug)

            game_exec = self.GetSelectedProgram()
            game_log = playonlinux.getLog(game_exec)
            if (game_log):
                i += 1
                self.menuGaucheAddLink("pol_prgm_reportproblem", _("Send a feedback"), i,
                                       Variables.playonlinux_env + "/resources/images/menu/bug.png", self.sendfeedback)

            i += 1
            self.menuGaucheAddLink("pol_prgm_configure", _("Configure"), i,
                                   Variables.playonlinux_env + "/resources/images/menu/run.png", self.Configure)
            i += 1
            self.menuGaucheAddLink("pol_prgm_shortcut", _("Create a shortcut"), i,
                                   Variables.playonlinux_env + "/resources/images/menu/shortcut.png", self.Package)
            i += 1
            self.menuGaucheAddLink("pol_prgm_adddir", _("Open the directory"), i,
                                   Variables.playonlinux_env + "/resources/images/menu/folder-wine.png",
                                   self.GoToAppDir)

            if (os.path.exists(os.environ["POL_USER_ROOT"] + "/configurations/manuals/" + shortcut)):
                i += 1
                self.menuGaucheAddLink("pol_prgm_readme", _("Read the manual"), i,
                                       Variables.playonlinux_env + "/resources/images/menu/manual.png", self.ReadMe)

            i += 1
            self.menuGaucheAddLink("pol_prgm_uninstall", _("Uninstall"), i,
                                   Variables.playonlinux_env + "/resources/images/menu/window-close.png",
                                   self.UninstallGame)

            self.linksfile = os.environ["POL_USER_ROOT"] + "/configurations/links/" + shortcut
            if (os.path.exists(self.linksfile)):
                self.linksc = open(self.linksfile, "r").read().split("\n")
                for line in self.linksc:
                    if ("|" in line):
                        line = line.split("|")
                        i += 1
                        if ("PROFILEBUTTON/" in line[0]):
                            line[0] = line[0].replace("PROFILEBUTTON/", "")

                        self.menuGaucheAddLink("url_" + str(i), line[0], i,
                                               Variables.playonlinux_env + "/resources/images/menu/star.png", None,
                                               line[1])

            icon = os.environ["POL_USER_ROOT"] + "/icones/full_size/" + shortcut

            if (os.path.exists(icon)):
                try:
                    self.bitmap = wx.Image(icon)
                    if (self.bitmap.GetWidth() >= 48):
                        self.bitmap.Rescale(48, 48, wx.IMAGE_QUALITY_HIGH)
                        self.bitmap = self.bitmap.ConvertToBitmap()
                        self.menuBitmap = wx.StaticBitmap(self.menu_gauche, id=-1, bitmap=self.bitmap,
                                                          pos=(left_pos, 20 + (i + 2) * 20))
                except:
                    pass

    def menuGaucheAddTitle(self, id, text, pos):
        self.menuElem[id] = wx.StaticText(self.menu_gauche, -1, text, pos=(5, 5 + pos * 20))
        self.menuElem[id].SetForegroundColour((0, 0, 0))  # For dark themes
        self.menuElem[id].SetFont(self.fontTitre)

    def menuGaucheAddLink(self, id, text, pos, image, evt, url=None):
        if (os.path.exists(image)):
            menu_icone = image
        else:
            menu_icone = Variables.playonlinux_env + "/etc/playonlinux.png"

        try:
            self.bitmap = wx.Image(menu_icone)
            self.bitmap.Rescale(16, 16, wx.IMAGE_QUALITY_HIGH)
            self.bitmap = self.bitmap.ConvertToBitmap()
            self.menuImage[id] = wx.StaticBitmap(self.menu_gauche, id=-1, bitmap=self.bitmap, pos=(10, 15 + pos * 20))

        except:
            pass

        if (url == None):
            self.menuElem[id] = wx.lib.hyperlink.HyperLinkCtrl(self.menu_gauche, 10000 + pos, text,
                                                               pos=(35, 15 + pos * 20))
            self.menuElem[id].AutoBrowse(False)
        else:
            self.menuElem[id] = wx.lib.hyperlink.HyperLinkCtrl(self.menu_gauche, 10000 + pos, text,
                                                               pos=(35, 15 + pos * 20))
            self.menuElem[id].setURL(url)

        self.menuElem[id].SetColours(wx.Colour(0, 0, 0), wx.Colour(0, 0, 0), wx.Colour(0, 0, 0))
        self.menuElem[id].UpdateLink(True)
        # self.menuElem[id].SetVisited(False)
        # self.menuElem[id].SetNormalColour(wx.Colour(0,0,0))
        # self.menuElem[id].SetVisitedColour(wx.Colour(0,0,0))
        # self.menuElem[id].SetHoverColour(wx.Colour(100,100,100))

        if (evt != None):
            wx.lib.hyperlink.EVT_HYPERLINK_LEFT(self, 10000 + pos, evt)

    def donate(self, event):
        if (os.environ["POL_OS"] == "Mac"):
            webbrowser.open("http://www.playonmac.com/en/donate.html")
        else:
            webbrowser.open("http://www.playonlinux.com/en/donate.html")

    def Reload(self, event):
        self.games = os.listdir(Variables.playonlinux_rep + "shortcuts/")
        self.games.sort(key=str.upper)

        try:
            self.games.remove(".DS_Store")
        except:
            pass

        self.list_game.DeleteAllItems()
        self.images.RemoveAll()
        root = self.list_game.AddRoot("")
        self.i = 0
        if (self.iconSize <= 32):
            self.iconFolder = "32"
        else:
            self.iconFolder = "full_size"
        for game in self.games:  # METTRE EN 32x32
            if (self.searchbox.GetValue().encode("utf-8", "replace").lower() in game.lower()):
                if (not os.path.isdir(Variables.playonlinux_rep + "/shortcuts/" + game)):
                    if (os.path.exists(Variables.playonlinux_rep + "/icones/" + self.iconFolder + "/" + game)):
                        file_icone = Variables.playonlinux_rep + "/icones/" + self.iconFolder + "/" + game
                    else:
                        file_icone = Variables.playonlinux_env + "/etc/playonlinux.png"

                    try:
                        self.bitmap = wx.Image(file_icone)
                        self.bitmap.Rescale(self.iconSize, self.iconSize, wx.IMAGE_QUALITY_HIGH)
                        self.bitmap = self.bitmap.ConvertToBitmap()
                        self.images.Add(self.bitmap)
                    except:
                        pass

                    item = self.list_game.AppendItem(root, game, self.i)
                    self.i += 1
        self.generate_menu(None)

        if (os.environ["POL_OS"] == "Mac"):
            self.playTool.Enable(False)
            self.stopTool.Enable(False)
            self.removeTool.Enable(False)

    def RConfigure(self, function_to_run, *args):
        """Starts polconfigurator remotely."""
        game_exec = self.GetSelectedProgram()
        if game_exec != "":
            subprocess.Popen(
                ["bash", Variables.playonlinux_env + "/bash/polconfigurator", game_exec, function_to_run] + list(args))
        else:
            wx.MessageBox(_("Please select a program."), os.environ["APPLICATION_TITLE"])

    def Options(self, event):
        onglet = event.GetId()
        try:
            self.optionFrame.SetFocus()
        except:
            self.optionFrame = options.MainWindow(self, -1, _("{0} settings").format(os.environ["APPLICATION_TITLE"]),
                                                  2)
            if (onglet == 211):
                self.optionFrame = options.MainWindow(self, -1,
                                                      _("{0} settings").format(os.environ["APPLICATION_TITLE"]), 0)
            if (onglet == 214):
                self.optionFrame = options.MainWindow(self, -1,
                                                      _("{0} settings").format(os.environ["APPLICATION_TITLE"]), 1)
            self.optionFrame.Center(wx.BOTH)
            self.optionFrame.Show(True)

    def killall(self, event):
        subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/killall"])

    def Executer(self, event):
        subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/expert/Executer"])

    def BugReport(self, event):
        try:
            self.debugFrame.Show()
            self.debugFrame.SetFocus()
        except:
            self.debugFrame = debug.MainWindow(None, -1, _("{0} debugger").format(os.environ["APPLICATION_TITLE"]))
            self.debugFrame.Center(wx.BOTH)
            self.debugFrame.Show()

    def POLOnline(self, event):
        subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/playonlinux_online"])

    def PCCd(self, event):
        subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/read_pc_cd"])

    def PolShell(self, event):
        # Variables.run_x_server()
        subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/expert/PolShell"])

    def Configure(self, event):
        game_exec = self.GetSelectedProgram()
        try:
            self.configureFrame.Show(True)
            self.configureFrame.SetFocus()
            if (game_exec != ""):
                self.configureFrame.change_program(game_exec, False)

        except:
            if (game_exec == ""):
                self.configureFrame = ConfigureWindow.ConfigureWindow(self, -1, _("{0} configuration").format(
                    os.environ["APPLICATION_TITLE"]), "default", True)
            else:
                self.configureFrame = ConfigureWindow.ConfigureWindow(self, -1, _("{0} configuration").format(
                    os.environ["APPLICATION_TITLE"]), game_exec.decode("utf-8", "replace"), False)

            self.configureFrame.Center(wx.BOTH)
            self.configureFrame.Show(True)

        # subprocess.Popen(["bash", Variables.playonlinux_env+"/bash/polconfigurator", game_exec])

    def Package(self, event):
        game_exec = self.GetSelectedProgram()
        subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/make_shortcut", game_exec])

    def UninstallGame(self, event):
        game_exec = self.GetSelectedProgram()
        if game_exec != "":
            subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/uninstall", game_exec])
        else:
            wx.MessageBox(_("Please select a program."), os.environ["APPLICATION_TITLE"])

    def PolVaultSaveGame(self, event):
        game_exec = self.GetSelectedProgram()
        if game_exec != "":
            subprocess.Popen(
                ["bash", Variables.playonlinux_rep + "plugins/PlayOnLinux Vault/scripts/menu", "--app", game_exec])
        else:
            wx.MessageBox(_("Please select a program."), os.environ["APPLICATION_TITLE"])

    def InstallMenu(self, event):
        try:
            self.installFrame.Show(True)
            self.installFrame.SetFocus()
        except:
            self.installFrame = InstallWindow(self, -1, _('{0} install menu').format(os.environ["APPLICATION_TITLE"]))
            self.installFrame.Center(wx.BOTH)
            self.installFrame.Show(True)

    def WineVersion(self, event):
        try:
            self.wversion.Show()
            self.wversion.SetFocus()
        except:
            self.wversion = wver.WineVersionsWindow(None, -1,
                                                    _('{0} wine versions manager').format(os.environ["APPLICATION_TITLE"]))
            self.wversion.Center(wx.BOTH)
            self.wversion.Show(True)

    def GetSelectedProgram(self):
        return self.list_game.GetItemText(self.list_game.GetSelection()).encode("utf-8", "replace")

    def Run(self, event, s_debug=False):

        game_exec = self.GetSelectedProgram()
        game_prefix = playonlinux.getPrefix(game_exec)

        if (s_debug == False):
            playonlinux.SetDebugState(game_exec, game_prefix, False)

        if (os.path.exists(os.environ["POL_USER_ROOT"] + "/wineprefix/" + game_prefix)):
            if (game_exec != ""):
                if (playonlinux.GetDebugState(game_exec)):
                    try:
                        self.debugFrame.analyseReal(0, game_prefix)
                        self.debugFrame.Show()
                        self.debugFrame.SetFocus()
                    except:
                        self.debugFrame = debug.MainWindow(None, -1,
                                                           _("{0} debugger").format(os.environ["APPLICATION_TITLE"]),
                                                           game_prefix, 0)
                        self.debugFrame.Center(wx.BOTH)
                        self.debugFrame.Show()

                subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/run_app", game_exec])
            else:
                wx.MessageBox(_("Please select a program."), os.environ["APPLICATION_TITLE"])
        else:
            wx.MessageBox(
                _("The virtual drive associated with {0} ({1}) does no longer exists.").format(game_exec, game_prefix),
                os.environ["APPLICATION_TITLE"])

    def RunDebug(self, event):
        game_exec = self.GetSelectedProgram()
        game_prefix = playonlinux.getPrefix(game_exec)
        playonlinux.SetDebugState(game_exec, game_prefix, True)
        self.Run(self, True)

    def sendfeedback(self, event):
        game_exec = self.GetSelectedProgram()
        game_log = str(playonlinux.getLog(game_exec))
        if game_log:
            playonlinux.POL_Open(
                "http://www." + os.environ["POL_DNS"] + "/repository/feedback.php?script=" + urllib.quote_plus(
                    game_log))

    def POLDie(self):
        for pid in self.registeredPid:
            try:
                os.kill(-pid, signal.SIGKILL)
            except OSError:
                pass
            try:
                os.kill(pid, signal.SIGKILL)
            except OSError:
                pass
        app.POLServer.closeServer()
        os._exit(0)

    def POLRestart(self):
        for pid in self.registeredPid:
            try:
                os.kill(-pid, signal.SIGKILL)
            except OSError:
                pass
            try:
                os.kill(pid, signal.SIGKILL)
            except OSError:
                pass
        app.POLServer.closeServer()
        os._exit(63)  # Restart code

    def ForceClose(self, signal, frame):  # Catch SIGINT
        print "\nCtrl+C pressed. Killing all processes..."
        self.POLDie()

    def ClosePol(self, event):
        pids = []
        for pid in self.registeredPid:
            try:
                os.kill(pid, 0)
                pid_exists = True
                pids.append(pid)
            except OSError:
                pid_exists = False
            print "Registered PID: %d (%s)" % (pid, 'Present' if pid_exists else 'Missing')
        self.registeredPid = pids

        if (playonlinux.GetSettings(
                "DONT_ASK_BEFORE_CLOSING") == "TRUE" or self.registeredPid == [] or wx.YES == wx.MessageBox(
                _('Are you sure you want to close all {0} windows?').format(os.environ["APPLICATION_TITLE"]).decode(
                        "utf-8", "replace"), os.environ["APPLICATION_TITLE"], style=wx.YES_NO | wx.ICON_QUESTION)):
            self.SizeToSave = self.GetSize()
            self.PositionToSave = self.GetPosition()
            # Save size and position
            playonlinux.SetSettings("MAINWINDOW_WIDTH", str(self.SizeToSave[0]))
            playonlinux.SetSettings("MAINWINDOW_HEIGHT", str(self.SizeToSave[1] - Variables.windows_add_playonmac * 56))
            playonlinux.SetSettings("MAINWINDOW_X", str(self.PositionToSave[0]))
            playonlinux.SetSettings("MAINWINDOW_Y", str(self.PositionToSave[1]))

            self._mgr.Destroy()

            self.POLDie()
        return None

    def About(self, event):
        self.aboutBox = wx.AboutDialogInfo()
        if (os.environ["POL_OS"] != "Mac"):
            self.aboutBox.SetIcon(wx.Icon(Variables.playonlinux_env + "/etc/playonlinux.png", wx.BITMAP_TYPE_ANY))

        self.aboutBox.SetName(os.environ["APPLICATION_TITLE"])
        self.aboutBox.SetVersion(Variables.version)
        self.aboutBox.SetDescription(_("Run your Windows programs on " + os.environ["POL_OS"] + " !"))
        self.aboutBox.SetCopyright("© 2007-2018 " + _("PlayOnLinux and PlayOnMac team\nUnder GPL licence version 3"))
        self.aboutBox.AddDeveloper(_("Developer and Website: ") + "Tinou (Pâris Quentin), MulX (Petit Aymeric)")
        self.aboutBox.AddDeveloper(_("Scriptors: ") + "GNU_Raziel")
        self.aboutBox.AddDeveloper(_("Packager: ") + "MulX (Petit Aymeric), Tinou (Pâris Quentin)")
        self.aboutBox.AddDeveloper(_("Icons:") + "Faenza-Icons http://tiheum.deviantart.com/art/Faenza-Icons-173323228")
        self.aboutBox.AddDeveloper(
            _("The following people contributed to this program: ") + "kiplantt, Salvatos, Minchul")
        self.aboutBox.AddTranslator(_("Translations:"))
        self.aboutBox.AddTranslator(_("Read TRANSLATORS file"))

        if (os.environ["POL_OS"] == "Mac"):
            self.aboutBox.SetWebSite("http://www.playonmac.com")
        else:
            self.aboutBox.SetWebSite("http://www.playonlinux.com")
        wx.AboutBox(self.aboutBox)


class PlayOnLinuxApp(wx.App):
    def OnInit(self):
        lng.iLang()
        close = False
        exe_present = False

        subprocess.call(["bash", Variables.playonlinux_env + "/bash/startup"])
        self.systemCheck()

        for f in sys.argv[1:]:
            self.MacOpenFile(f)
            if (".exe" in f or ".EXE" in f):
                exe_present = True
            close = True

        if (close == True and exe_present == False):
            os._exit(0)

        self.SetClassName(os.environ["APPLICATION_TITLE"])
        self.SetAppName(os.environ["APPLICATION_TITLE"])

        self.frame = MainWindow(None, -1, os.environ["APPLICATION_TITLE"])
        # Gui Server
        self.POLServer = gui_server.gui_server(self.frame)
        self.POLServer.start()

        i = 0
        while (os.environ["POL_PORT"] == "0"):
            time.sleep(0.01)
            if (i >= 300):
                wx.MessageBox(_("{0} is not able to start PlayOnLinux Setup Window server.").format(
                    os.environ["APPLICATION_TITLE"]), _("Error"))
                os._exit(0)
                break
            i += 1
        subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/startup_after_server"])

        self.SetTopWindow(self.frame)
        self.frame.Show(True)

        return True

    def _executableFound(self, executable):
        devnull = open('/dev/null', 'wb')
        try:
            returncode = subprocess.call(["which", executable], stdout=devnull)
            return (returncode == 0)
        except:
            return False

    def _singleCheck(self, executable, package, fatal):
        if not self._executableFound(executable):
            message = _("{0} cannot find {1}")
            if package is not None:
                message += _(" (from {2})")

            if fatal:
                verdict = _("You need to install it to continue")
            else:
                verdict = _("You should install it to use {0}")

            wx.MessageBox(
                ("%s\n\n%s" % (message, verdict)).format(os.environ["APPLICATION_TITLE"], executable, package),
                _("Error"))

            if fatal:
                os._exit(0)

    def singleCheck(self, executable, package=None):
        self._singleCheck(executable, package, False)

    def singleCheckFatal(self, executable, package=None):
        self._singleCheck(executable, package, True)

    def systemCheck(self):
        #### Root uid check
        if (os.popen("id -u").read() == "0\n" or os.popen("id -u").read() == "0"):
            wx.MessageBox(_("{0} is not supposed to be run as root. Sorry").format(os.environ["APPLICATION_TITLE"]),
                          _("Error"))
            os._exit(0)

        #### 32 bits OpenGL check
        try:
            returncode = subprocess.call([Variables.playonlinux_env + "/bash/check_gl", "x86"])
        except:
            returncode = 255
        if (os.environ["POL_OS"] == "Linux" and returncode != 0):
            wx.MessageBox(_(
                "{0} is unable to find 32bits OpenGL libraries.\n\nYou might encounter problem with your games").format(
                os.environ["APPLICATION_TITLE"]), _("Error"))
            os.environ["OpenGL32"] = "0"
        else:
            os.environ["OpenGL32"] = "1"

        #### 64 bits OpenGL check
        if (os.environ["AMD64_COMPATIBLE"] == "True"):
            try:
                returncode = subprocess.call([Variables.playonlinux_env + "/bash/check_gl", "amd64"])
            except:
                returncode = 255
        if (os.environ["AMD64_COMPATIBLE"] == "True" and os.environ["POL_OS"] == "Linux" and returncode != 0):
            wx.MessageBox(_(
                "{0} is unable to find 64bits OpenGL libraries.\n\nYou might encounter problem with your games").format(
                os.environ["APPLICATION_TITLE"]), _("Error"))
            os.environ["OpenGL64"] = "0"
        else:
            os.environ["OpenGL64"] = "1"

        #### Filesystem check
        if (os.environ["POL_OS"] == "Linux"):
            try:
                returncode = subprocess.call([Variables.playonlinux_env + "/bash/check_fs"])
            except:
                returncode = 255
            if (os.environ["POL_OS"] == "Linux" and returncode != 0):
                wx.MessageBox(_(
                    "Your filesystem might prevent {0} from running correctly.\n\nPlease open {0} in a terminal to get more details").format(
                    os.environ["APPLICATION_TITLE"]), _("Error"))

        if (os.environ["DEBIAN_PACKAGE"] == "FALSE"):
            if (playonlinux.GetSettings("SEND_REPORT") == ""):
                if (wx.YES == wx.MessageBox(_(
                        'Do you want to help {0} to make a compatibility database?\n\nIf you click yes, the following things will be sent to us anonymously the first time you run a Windows program:\n\n- Your graphic card model\n- Your OS version\n- If graphic drivers are installed or not.\n\n\nThese information will be very precious for us to help people.').format(
                        os.environ["APPLICATION_TITLE"]).decode("utf-8", "replace"), os.environ["APPLICATION_TITLE"],
                                            style=wx.YES_NO | wx.ICON_QUESTION)):
                    playonlinux.SetSettings("SEND_REPORT", "TRUE")
                else:
                    playonlinux.SetSettings("SEND_REPORT", "FALSE")

        #### Other import checks
        self.singleCheckFatal("nc", package="Netcat")
        self.singleCheckFatal("tar")
        self.singleCheckFatal("cabextract")
        self.singleCheckFatal("convert", package="ImageMagick")
        self.singleCheckFatal("wget", package="Wget")
        self.singleCheckFatal("curl", package="cURL")

        self.singleCheckFatal("gpg", package="GnuPG")

        # FIXME: now that PoL can use a range of terminal emulators, xterm is no longer
        # a hard dependency
        if (os.environ["DEBIAN_PACKAGE"] == "FALSE" and os.environ["POL_OS"] != "Mac"):
            self.singleCheck("xterm")
        self.singleCheck("gettext.sh", package="gettext")  # gettext-base on Debian
        self.singleCheck("icotool", package="icoutils")
        self.singleCheck("wrestool", package="icoutils")
        self.singleCheck("wine", package="Wine")
        self.singleCheck("unzip", package="InfoZIP")
        self.singleCheck("jq", package="jq")
        self.singleCheck("7z", package="P7ZIP full")  # p7zip-full on Debian
        if (os.environ["POL_OS"] == "FreeBSD"):
            self.singleCheck("gsed", package="GNU Sed")

    def BringWindowToFront(self):
        try:  # it's possible for this event to come when the frame is closed
            self.GetTopWindow().Raise()
        except:
            pass

    def MacOpenFile(self, filename):
        file_extension = string.split(filename, ".")
        file_extension = file_extension[len(file_extension) - 1]
        if (file_extension == "desktop"):  # Un raccourcis Linux
            content = open(filename, "r").readlines()
            i = 0
            while (i < len(content)):
                # wx.MessageBox(content[i], "PlayOnLinux", wx.OK)

                if ("Path=" in content[i]):
                    cd_app = content[i].replace("Path=", "").replace("\n", "")
                if ("Exec=" in content[i]):
                    exec_app = content[i].replace("Exec=", "").replace("\n", "")
                i += 1
            if (":\\\\\\\\" in exec_app):
                exec_app = exec_app.replace("\\\\", "\\")
            try:
                subprocess.Popen(shlex.split(exec_app), cwd=cd_app)
            except:
                pass

        elif (file_extension == "exe" or file_extension == "EXE"):
            subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/run_exe", filename])

        elif (file_extension == "pol" or file_extension == "POL"):
            if (wx.YES == wx.MessageBox(
                    _('Are you sure you want to  want to install {0} package?').format(filename).decode("utf-8",
                                                                                                        "replace"),
                    os.environ["APPLICATION_TITLE"], style=wx.YES_NO | wx.ICON_QUESTION)):
                subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/playonlinux-pkg", "-i", filename])
        else:
            playonlinux.open_document(filename, file_extension.lower())

    def MacOpenURL(self, url):
        if (os.environ["POL_OS"] == "Mac" and not "playonmac://" in url):
            wx.MessageBox(_("You are trying to open a script design for {0}! It might not work as expected").format(
                "PlayOnLinux or PlayOnBSD"), os.environ["APPLICATION_TITLE"])
        if (os.environ["POL_OS"] == "Linux" and not "playonlinux://" in url):
            wx.MessageBox(_("You are trying to open a script design for {0}! It might not work as expected").format(
                "PlayOnMac or PlayOnBSD"), os.environ["APPLICATION_TITLE"])
        if (os.environ["POL_OS"] == "FreeBSD" and not "playonbsd://" in url):
            wx.MessageBox(_("You are trying to open a script design for {0}! It might not work as expected").format(
                "PlayOnMac or PlayOnLinux"), os.environ["APPLICATION_TITLE"])

        subprocess.Popen(["bash", Variables.playonlinux_env + "/bash/playonlinux-url_handler", url])

    def MacReopenApp(self):
        # sys.exit()
        self.BringWindowToFront()


# Idea taken from flacon
def setSigchldHandler():
    signal.signal(signal.SIGCHLD, handleSigchld)
    if hasattr(signal, 'siginterrupt'):
        signal.siginterrupt(signal.SIGCHLD, False)


def handleSigchld(number, frame):
    # Apparently some UNIX systems automatically resent the SIGCHLD
    # handler to SIG_DFL.  Reset it just in case.
    setSigchldHandler()


setSigchldHandler()
lng.Lang()

wx.Log_EnableLogging(False)

app = PlayOnLinuxApp(redirect=False)
app.MainLoop()
# sys.exit(0)



import wx, os
import lib.Variables as Variables

class PlayOnLinuxWindow(wx.Frame):
    def __init__(self, parent, windowId, title = None, size = None, style = wx.DEFAULT_FRAME_STYLE):
        wx.Frame.__init__(self, parent, windowId, title, size = size, style = style)
        self.SetIcon(wx.Icon(Variables.playonlinux_env + "/etc/playonlinux.png", wx.BITMAP_TYPE_ANY))
        self.parent = parent

        if os.environ["POL_OS"] == "Mac":
            self.fontText = wx.Font(10, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL,False, "", wx.FONTENCODING_DEFAULT)
            self.fontTitle = wx.Font(14, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD, False, "", wx.FONTENCODING_DEFAULT)
        else :
            self.fontText = wx.Font(8, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL,False, "", wx.FONTENCODING_DEFAULT)
            self.fontTitle = wx.Font(10, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD, False, "", wx.FONTENCODING_DEFAULT)



import wx

from lib import Variables


## FIXME: Refactor this code later
class WineVersionsNotebook(wx.Notebook):
    def __init__(self, parent):
        self.notebook = wx.Notebook.__init__(self, parent, -1)
        self.images_onglets = wx.ImageList(16, 16)
        self.images_onglets.Add(wx.Bitmap(Variables.playonlinux_env + "/etc/onglet/wine.png"))
        self.SetImageList(self.images_onglets)

        self.sizers = {}
        self.imagesapps = {}
        self.imagesapps_i = {}
        self.availableWineVersionsTreeSelector = {}
        self.waitPanel = {}
        self.installedWineVersionsTreeSelector = {}
        self.button_rm = {}
        self.button_in = {}
        self.boxSizers = {}
        self.panels = {}

    def calculateEventDelta(self, arch):
        if (arch == "amd64"):
            return 100
        else:
            return 0

    def _createInstalledWineVersionsPanel(self, arch):
        installedWineVersionsPanel = wx.Panel(self.panels[arch], -1)
        installedWineVersionsSizer = wx.BoxSizer(wx.VERTICAL)
        installedWineVersionsPanel.SetSizer(installedWineVersionsSizer)
        self.boxSizers[arch].Add(installedWineVersionsPanel, 3, wx.EXPAND)
        installedWineVersionsSizer.AddSpacer(10)
        installedWineVersionsSizer.Add(wx.StaticText(installedWineVersionsPanel, -1, _("Installed Wine versions: ")), 0)
        installedWineVersionsSizer.AddSpacer(10)
        self.installedWineVersionsTreeSelector[arch] = wx.TreeCtrl(installedWineVersionsPanel, 107 + self.calculateEventDelta(arch),
                                                                   style=wx.TR_HIDE_ROOT | wx.TR_FULL_ROW_HIGHLIGHT | Variables.widget_borders)
        self.installedWineVersionsTreeSelector[arch].SetImageList(self.imagesapps_i[arch])
        self.installedWineVersionsTreeSelector[arch].SetSpacing(0)
        installedWineVersionsSizer.Add(self.installedWineVersionsTreeSelector[arch], 1, wx.EXPAND)
        installedWineVersionsSizer.AddSpacer(10)

    def _createAvailableWineVersionsPanel(self, arch):
        availableWineVersionsPanel = wx.Panel(self.panels[arch] , -1)
        availableWineVersionsSizer = wx.BoxSizer(wx.VERTICAL)
        availableWineVersionsPanel.SetSizer(availableWineVersionsSizer)
        self.boxSizers[arch].Add(availableWineVersionsPanel, 3, wx.EXPAND)
        availableWineVersionsSizer.AddSpacer(10)
        availableWineVersionsSizer.Add(wx.StaticText(availableWineVersionsPanel, -1, _("Available Wine versions: "), (5, 10)), 0)
        availableWineVersionsSizer.AddSpacer(10)
        self.availableWineVersionsTreeSelector[arch] = wx.TreeCtrl(availableWineVersionsPanel, 106 + self.calculateEventDelta(arch),
                                                                   style=wx.TR_HIDE_ROOT | wx.TR_FULL_ROW_HIGHLIGHT | Variables.widget_borders,
                                                                   size=(320, 300), pos=(10, 35))
        self.availableWineVersionsTreeSelector[arch].SetImageList(self.imagesapps[arch])
        self.availableWineVersionsTreeSelector[arch].SetSpacing(0)
        availableWineVersionsSizer.Add(self.availableWineVersionsTreeSelector[arch], 1, wx.EXPAND)
        availableWineVersionsSizer.AddSpacer(10)

    def _createButtonPanel(self, arch):
        buttonsPanel = wx.Panel(self.panels[arch] , -1)
        buttonsSizer = wx.BoxSizer(wx.VERTICAL)
        buttonsPanel.SetSizer(buttonsSizer)

        self.button_rm[arch] = wx.Button(buttonsPanel, 108 + self.calculateEventDelta(arch), "<")
        self.button_in[arch] = wx.Button(buttonsPanel, 109 + self.calculateEventDelta(arch), ">")

        buttonsSizer.AddStretchSpacer()
        buttonsSizer.Add(self.button_rm[arch], 0, wx.CENTER)
        buttonsSizer.AddSpacer(10)
        buttonsSizer.Add(self.button_in[arch], 0, wx.CENTER)
        buttonsSizer.AddStretchSpacer()

        self.button_rm[arch].Enable(False)
        self.button_in[arch].Enable(False)

        self.boxSizers[arch].Add(buttonsPanel, 1, wx.EXPAND)

    def liste_versions(self, arch="x86"):
        self.panels[arch] = wx.Panel(self, -1)

        self.boxSizers[arch] = wx.BoxSizer(wx.HORIZONTAL)
        self.panels[arch].SetSizer(self.boxSizers[arch])
        self.imagesapps[arch] = wx.ImageList(22, 22)
        self.imagesapps_i[arch] = wx.ImageList(22, 22)

        self.boxSizers[arch].AddSpacer(10)
        self._createAvailableWineVersionsPanel(arch)
        self._createButtonPanel(arch)
        self._createInstalledWineVersionsPanel(arch)
        self.boxSizers[arch].AddSpacer(10)

        self.AddPage(self.panels[arch], _("Wine versions") + " (" + arch + ")", imageId=0)



from lib import Variables


class PackageList:
    def __init__(self):
        self.available_packages = []
        self.loadList()

    def loadList(self):
        try:
            self.available_packages = open(Variables.playonlinux_rep + "/configurations/listes/POL_Functions",
                                           "r").read()
        except IOError as e:  # File does not exits  it will be created when pol is updated
            pass

    def getList(self):
        return self.available_packages

    def getCutList(self):
        clist = self.available_packages.split("\n")
        flist = []
        for key in clist:
            if ("POL_Install" in key):
                flist.append(key)
        return flist

    def getParsedList(self):
        clist = self.getCutList()
        flist = []
        for key in clist:
            flist.append(PackageList.getNameFromPackageLine(key))
        return flist

    def getNameFromId(self, id):
        return self.getParsedList()[id]

    def getPackageFromName(self, selectedPackage):
        broken = False
        for key in self.getCutList():
            key_split = key.split(":")
            try:
                if (key_split[1] == selectedPackage):  # We found it
                    selectedPackage = key_split[0]
                    broken = True
                    break

            except IndexError, e:  # Index error : There is no ':' in the line, so the content of the line is the package we want to install. No need to continue
                broken = True
                break

        if (broken == False):
            selectedPackage = "POL_Install_" + selectedPackage
        return selectedPackage

    @staticmethod
    def getNameFromPackageLine(package):
        try:
            realName = package.split(":")[1].replace("POL_Install_", "")
        except IndexError, e:
            realName = package.replace("POL_Install_", "")
        return realName


import os
import string

from lib import Variables

def fetchUserOS():
    if (os.environ["POL_OS"] == "Mac"):
        return "darwin"
    elif (os.environ["POL_OS"] == "Linux"):
        return "linux"
    else:
        return "freebsd"

def GetWineVersion(game):
    cfile = Variables.playonlinux_rep + "shortcuts/" + game
    fichier = open(cfile, "r").readlines()
    i = 0
    line = ""
    while (i < len(fichier)):
        fichier[i] = fichier[i].replace("\n", "")
        if ("PATH=" in fichier[i] and "WineVersions" in fichier[i]):
            line = fichier[i].replace("//", "/")
        i += 1

    if line == "":
        version = "System"
    else:
        version = line.replace("PATH=", "").replace("\"", "").replace(Variables.playonlinux_rep, "").replace("//", "/")
        version = string.split(version, "/")
        version = version[1]

    return version


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2008 Pâris Quentin

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import codecs
import os
import string
import subprocess

import wx
import wx.animate
import wx.html
import wx.lib.hyperlink
from wx.lib.ClickableHtmlWindow import PyClickableHtmlWindow

import lib.Variables as Variables
import lib.playonlinux as playonlinux
from install.DescriptionFetcher import DescriptionFetcher
from install.MiniatureWindow import MiniatureWindow
from ui.PlayOnLinuxWindow import PlayOnLinuxWindow


class InstallWindow(PlayOnLinuxWindow):
    def addCat(self, name, icon, iid):
        espace = 80
        if (os.environ["POL_OS"] == "Mac"):
            offset = 10
        else:
            offset = 2

        self.cats_icons[name] = wx.BitmapButton(self.installWindowHeader, 2000 + iid, wx.Bitmap(icon), (0, 0),
                                                style=wx.NO_BORDER)

        self.cats_links[name] = wx.lib.hyperlink.HyperLinkCtrl(self.installWindowHeader, 3000 + iid, name, pos=(0, 52))
        mataille = self.cats_links[name].GetSize()[0]

        mataille2 = self.cats_icons[name].GetSize()[0]
        image_pos = (espace - mataille2) / 2 + espace * iid;

        self.cats_links[name].SetPosition((espace * iid + (espace - mataille / 1.3) / 2, 47))
        self.cats_icons[name].SetPosition((image_pos, offset))

        wx.lib.hyperlink.EVT_HYPERLINK_LEFT(self, 3000 + iid, self.AddApps)
        wx.EVT_BUTTON(self, 2000 + iid, self.AddApps)

        self.cats_links[name].SetColours(wx.Colour(0, 0, 0), wx.Colour(0, 0, 0), wx.Colour(0, 0, 0))
        self.cats_links[name].AutoBrowse(False)
        self.cats_links[name].UpdateLink(True)
        self.cats_links[name].SetUnderlines(False, False, False)

        self.cats_links[name].SetFont(self.fontText)
        self.cats_links[name].SetBackgroundColour((255, 255, 255))

    def setWaitState(self, isWaiting):
        if isWaiting:
            self.installWindowCategoryContentPanel.Hide()
            self.panelWait.Show()
            self.manualInstall.Raise()
            self.animation_wait.Play()
            self.installButton.Hide()
            self.updateButton.Hide()
        else:
            self.installWindowCategoryContentPanel.Show()
            self.manualInstall.Raise()
            self.panelWait.Hide()
            self.animation_wait.Stop()
            self.installButton.Show()
            self.updateButton.Show()

        self.Layout()

    def _createHeader(self):
        self.installWindowHeader = wx.Panel(self, -1, size=(802, 69))
        self.installWindowHeader.SetBackgroundColour((255, 255, 255))
        self.windowSizer.Add(self.installWindowHeader, 0, wx.EXPAND)
        self.addCat(_("Accessories"),
                    Variables.playonlinux_env + "/resources/images/install/32/applications-accessories.png", 0)
        self.addCat(_("Development"),
                    Variables.playonlinux_env + "/resources/images/install/32/applications-development.png", 1)
        self.addCat(_("Education"), Variables.playonlinux_env + "/resources/images/install/32/applications-science.png",
                    2)
        self.addCat(_("Games"), Variables.playonlinux_env + "/resources/images/install/32/applications-games.png", 3)
        self.addCat(_("Graphics"), Variables.playonlinux_env + "/resources/images/install/32/applications-graphics.png",
                    4)
        self.addCat(_("Internet"), Variables.playonlinux_env + "/resources/images/install/32/applications-internet.png",
                    5)
        self.addCat(_("Multimedia"),
                    Variables.playonlinux_env + "/resources/images/install/32/applications-multimedia.png", 6)
        self.addCat(_("Office"), Variables.playonlinux_env + "/resources/images/install/32/applications-office.png", 7)
        self.addCat(_("Other"), Variables.playonlinux_env + "/resources/images/install/32/applications-other.png", 8)
        self.addCat(_("Patches"), Variables.playonlinux_env + "/resources/images/install/32/view-refresh.png", 9)

    def _createBody(self):
        self.installWindowBodyPanel = wx.Panel(self, -1)
        self.windowSizer.Add(self.installWindowBodyPanel, 1, wx.EXPAND)
        self.installWindowBodySizer = wx.BoxSizer(wx.VERTICAL)
        self.installWindowBodyPanel.SetSizer(self.installWindowBodySizer)

    def _createWaitPanel(self):
        self.panelWait = wx.Panel(self.installWindowBodyPanel, -1)
        self.installWindowBodySizer.Add(self.panelWait, 1, wx.EXPAND)
        self.panelWait.Hide()
        ## FIXME: Remove those magic numbers
        self.animation_wait = wx.animate.GIFAnimationCtrl(self.panelWait, -1,
                                                          Variables.playonlinux_env + "/resources/images/install/wait.gif",
                                                          ((800 - 128) / 2, (550 - 128) / 2 - 71))
        self.percentageText = wx.StaticText(self.panelWait, -1, "", ((800 - 30) / 2, (550 - 128) / 2 + 128 + 10 - 71),
                                            wx.DefaultSize)
        self.percentageText.SetFont(self.fontTitle)
        self.timer = wx.Timer(self, 1)
        self.Bind(wx.EVT_TIMER, self.TimerAction, self.timer)
        self.timer.Start(200)

    def _createInstallWindowCategoryContentPanel(self):
        self.installWindowCategoryContentPanel = wx.Panel(self.installWindowBodyPanel, -1)

        self.installWindowBodySizer.Add(self.installWindowCategoryContentPanel, 1, wx.EXPAND)
        self.installWindowCategoryContentSizer = wx.BoxSizer(wx.VERTICAL)
        self.installWindowCategoryContentPanel.SetSizer(self.installWindowCategoryContentSizer)

    def _createFilterPanel(self):
        self.filterPanel = wx.Panel(self.installWindowCategoryContentPanel, -1)

        self.installWindowCategoryContentSizer.AddSpacer(10)
        self.installWindowCategoryContentSizer.Add(self.filterPanel, 0, wx.EXPAND)
        self.installWindowCategoryContentSizer.AddSpacer(10)

        filterSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.filterPanel.SetSizer(filterSizer)

        self.searchbox = wx.SearchCtrl(self.filterPanel, 110,)
        filtersCaption = wx.StaticText(self.filterPanel, -1, _("Include:"))
        self.testingChk = wx.CheckBox(self.filterPanel, 401)
        self.testingChk.SetValue(True)
        testingCaption = wx.StaticText(self.filterPanel, -1, _("Testing"))
        self.nocdChk = wx.CheckBox(self.filterPanel, 402)
        nocdCaption = wx.StaticText(self.filterPanel, -1, _("No-cd needed"))
        self.commercialChk = wx.CheckBox(self.filterPanel, 403)
        self.commercialChk.SetValue(True)
        commercialCaption = wx.StaticText(self.filterPanel, -1, _("Commercial"))
        self.panelStars = wx.Panel(self.filterPanel, -1)

        filterSizer.AddSpacer(10)
        filterSizer.Add(self.searchbox, 4, wx.EXPAND)
        filterSizer.AddSpacer(10)
        filterSizer.Add(filtersCaption, 0, wx.EXPAND)
        filterSizer.AddSpacer(10)
        filterSizer.Add(self.testingChk, 0, wx.EXPAND)
        filterSizer.Add(testingCaption, 0, wx.EXPAND)
        filterSizer.AddSpacer(10)
        filterSizer.Add(self.nocdChk, 0, wx.EXPAND)
        filterSizer.Add(nocdCaption, 0, wx.EXPAND)
        filterSizer.AddSpacer(10)
        filterSizer.Add(self.commercialChk, 0, wx.EXPAND)
        filterSizer.Add(commercialCaption, 0, wx.EXPAND)
        filterSizer.AddSpacer(20)
        filterSizer.Add(self.panelStars, 2, wx.EXPAND)

        filterSizer.AddSpacer(10)

    def _createAppNavigation(self):
        self.appNavigationPanel = wx.Panel(self.installWindowCategoryContentPanel, -1)
        self.installWindowCategoryContentSizer.Add(self.appNavigationPanel, 10, wx.EXPAND)
        self.appNavigationSizer = wx.BoxSizer(wx.HORIZONTAL)
        #
        self.appNavigationPanel.SetSizer(self.appNavigationSizer)

        self.imagesapps = wx.ImageList(22, 22)
        self.appsList = wx.TreeCtrl(self.appNavigationPanel, 106,
                                    style=wx.TR_HIDE_ROOT | wx.TR_FULL_ROW_HIGHLIGHT | Variables.widget_borders)
        self.appsList.SetImageList(self.imagesapps)
        self.appsList.SetSpacing(0)

        self.appNavigationSizer.AddSpacer(10)
        self.appNavigationSizer.Add(self.appsList, 4, wx.EXPAND, 5)
        self.appNavigationSizer.AddSpacer(10)

    def _createAppDescription(self):
        appDescriptionPanel = wx.Panel(self.appNavigationPanel, -1)
        self.appNavigationSizer.Add(appDescriptionPanel, 1, wx.EXPAND)
        self.appNavigationSizer.AddSpacer(10)

        appDescriptionSizer = wx.BoxSizer(wx.VERTICAL)

        self.descriptionContentHtmlBox = PyClickableHtmlWindow(appDescriptionPanel, 107, style=Variables.widget_borders)
        appDescriptionSizer.Add(self.descriptionContentHtmlBox, 1, wx.EXPAND)

        self.descriptionLoaderPanel = wx.Panel(appDescriptionPanel, -1, style=Variables.widget_borders)
        self.descriptionLoaderPanel.SetBackgroundColour((255, 255, 255))
        self.animation = wx.animate.GIFAnimationCtrl(self.descriptionLoaderPanel, -1, Variables.playonlinux_env + "/resources/images/install/wait_mini.gif", (90, 100))
        self.animation.Hide()
        self.descriptionLoaderPanel.Hide()

        self.image = wx.StaticBitmap(appDescriptionPanel, 108,
                                     wx.Bitmap(Variables.playonlinux_env + "/resources/images/pol_min.png"))
        self.image.Bind(wx.EVT_LEFT_DOWN, self.sizeUpScreen)

        appDescriptionSizer.Add(self.descriptionLoaderPanel, 1, wx.EXPAND)
        appDescriptionSizer.AddSpacer(10)
        appDescriptionSizer.Add(self.image, 0)
        appDescriptionPanel.SetSizer(appDescriptionSizer)

    def _createButtons(self):
        buttonsPanel = wx.Panel(self.installWindowBodyPanel, -1)
        buttonsSizer = wx.BoxSizer(wx.HORIZONTAL)
        buttonsPanel.SetSizer(buttonsSizer)

        self.installWindowBodySizer.AddSpacer(10)
        self.installWindowBodySizer.Add(buttonsPanel, 0, wx.EXPAND)
        self.installWindowBodySizer.AddSpacer(10)

        self.cancelButton = wx.Button(buttonsPanel, wx.ID_CLOSE, _("Cancel"))
        self.installButton = wx.Button(buttonsPanel, wx.ID_APPLY, _("Install"))
        self.updateButton = wx.Button(buttonsPanel, wx.ID_REFRESH, _("Refresh"))
        self.manualInstall = wx.lib.hyperlink.HyperLinkCtrl(buttonsPanel, 111, _("Install a non-listed program"))
        self.manualInstall.SetColours(wx.Colour(0, 0, 0), wx.Colour(0, 0, 0), wx.Colour(0, 0, 0))
        self.manualInstall.AutoBrowse(False)
        self.manualInstall.UpdateLink(True)

        buttonsSizer.AddSpacer(10)
        buttonsSizer.Add(self.manualInstall, 0)
        buttonsSizer.AddStretchSpacer()

        buttonsSizer.Add(self.updateButton, 0)
        buttonsSizer.AddSpacer(1)
        buttonsSizer.Add(self.cancelButton, 0)
        buttonsSizer.AddSpacer(1)
        buttonsSizer.Add(self.installButton, 0)
        buttonsSizer.AddSpacer(10)

    def __init__(self, parent, id, title):
        PlayOnLinuxWindow.__init__(self, parent, -1, title, size=(850, 550))
        self.cats_icons = {}
        self.cats_links = {}

        self.descriptionFetcher = DescriptionFetcher()

        ## Window
        self.windowSizer = wx.BoxSizer(wx.VERTICAL)
        self._createHeader()
        self._createBody()
        self.SetSizer(self.windowSizer)

        self._createInstallWindowCategoryContentPanel()
        self._createWaitPanel()

        # Filter panel
        self._createFilterPanel()

        # Apps Navigation
        self._createAppNavigation()
        self._createAppDescription()

        ## Buttons
        self._createButtons()

        self.live = 0
        self.openMin = False
        self.images_cat = wx.ImageList(22, 22)

        self.installButton.Enable(False)


        # wx.EVT_TREE_SEL_CHANGED(self, 105, self.AddApps)
        wx.EVT_TREE_SEL_CHANGED(self, 106, self.AppsDetails)
        wx.EVT_BUTTON(self, wx.ID_CLOSE, self.closeapp)
        wx.EVT_BUTTON(self, wx.ID_APPLY, self.installapp)
        wx.EVT_BUTTON(self, wx.ID_REFRESH, self.UpdatePol)
        wx.EVT_CLOSE(self, self.closeapp)
        wx.EVT_TREE_ITEM_ACTIVATED(self, 106, self.installapp)
        wx.EVT_TEXT(self, 110, self.search)
        wx.lib.hyperlink.EVT_HYPERLINK_LEFT(self, 111, self.manual)

        wx.EVT_CHECKBOX(self, 401, self.CheckBoxReload)
        wx.EVT_CHECKBOX(self, 402, self.CheckBoxReload)
        wx.EVT_CHECKBOX(self, 403, self.CheckBoxReload)

    def TimerAction(self, event):
        try:
            self.lasthtml_content
        except AttributeError:
            self.lasthtml_content = ""

        if self.lasthtml_content != self.descriptionFetcher.htmlContent:
            self.SetImg(self.descriptionFetcher.miniature)
            self.descriptionFetcher.miniature = self.descriptionFetcher.miniature_defaut

            self.lasthtml_content = self.descriptionFetcher.htmlContent
            if self.descriptionFetcher.htmlContent == "###WAIT###":
                self.animation.Show()
                self.animation.Play()
                self.descriptionLoaderPanel.Show()
                self.descriptionContentHtmlBox.Hide()
                self.Refresh()
            else:
                self.animation.Stop()
                self.descriptionContentHtmlBox.Show()
                self.animation.Hide()
                self.descriptionLoaderPanel.Hide()
                self.Refresh()
                self.descriptionContentHtmlBox.SetPage(self.descriptionFetcher.htmlContent)
            self.Layout()

        self.showStars(self.descriptionFetcher.stars)

        if self.openMin == True:
            if self.descriptionFetcher.med_miniature != None:
                self.miniatureWindow = MiniatureWindow(None, -1,
                                                       self.appsList.GetItemText(self.appsList.GetSelection()),
                                                       self.descriptionFetcher.med_miniature)
                self.miniatureWindow.Show()
                self.miniatureWindow.Center(wx.BOTH)
                self.openMin = False

    def closeapp(self, event):
        self.descriptionFetcher.thread_running = False
        self.Destroy()

    def manual(self, event):
        self.live = 1
        self.installapp(self)

    def showStars(self, stars):
        starWidth = 20
        self.panelStars.DestroyChildren()

        for i in xrange(int(stars)):
            wx.StaticBitmap(self.panelStars, -1,
                            wx.Bitmap(Variables.playonlinux_env + "/etc/star.png"),
                            (i * starWidth, 0), wx.DefaultSize)

    def UpdatePol(self, event):
        self.DelApps()
        self.Parent.updater.check()
        playonlinux.SetSettings("LAST_TIMESTAMP", "0")

    def installapp(self, event):
        if (self.live == 1):
            InstallApplication = "ExecLiveInstall"
        else:
            InstallApplication = self.appsList.GetItemText(self.appsList.GetSelection())

        if os.path.exists(Variables.playonlinux_rep + "/configurations/listes/search"):
            content = codecs.open(Variables.playonlinux_rep + "/configurations/listes/search", "r",
                                  "utf-8").read().split("\n")
            found = False
            for line in content:
                split = line.split("~")
                if (split[0] == InstallApplication):
                    found = True
                    break;
            if (found == True):
                if (len(split) <= 1):
                    self.UpdatePol(self)
                else:
                    if (split[1] == "1"):
                        wx.MessageBox(_(
                            "This program is currently in testing.\n\nIt might not work as expected. Your feedback, positive or negative, is specially important to improve this installer."),
                            _("Please read this"))
                    if (split[2] == "1"):
                        wx.MessageBox(_(
                            "This program contains a protection against copy (DRM) incompatible with emulation.\nThe only workaround is to use a \"no-cd\" patch, but since those can also be used for piracy purposes we won't give any support on this matter."),
                            _("Please read this"))

        subprocess.Popen(
            ["bash", Variables.playonlinux_env + "/bash/install", InstallApplication.encode("utf-8", "replace")])

        self.Destroy()
        return

    def search(self, event):
        self.apps = codecs.open(Variables.playonlinux_rep + "/configurations/listes/search", 'r', "utf-8")
        self.apps = self.apps.readlines()
        self.j = 0
        while (self.j < len(self.apps)):
            self.apps[self.j] = self.apps[self.j].replace("\n", "")
            self.j += 1

        self.j = 0
        self.k = 0
        self.user_search = self.searchbox.GetValue()
        self.search_result = []

        while (self.j < len(self.apps)):
            if (string.lower(self.user_search) in string.lower(self.apps[self.j])):
                self.search_result.append(self.apps[self.j])
                self.k = self.k + 1
            self.j = self.j + 1

        if (len(self.user_search) < 2 or "~" in self.user_search):
            self.search_result = []
        self.user_search_cut = string.split(self.user_search, ":")
        if (len(self.user_search_cut) > 1):
            if (self.user_search_cut[0] == "get" and self.user_search_cut[1].isdigit()):
                self.search_result.append(self.user_search)

        if (self.user_search != ""):
            self.WriteApps(self.search_result)
        else:
            self.DelApps()

    def EraseDetails(self):
        self.descriptionContentHtmlBox.SetValue("")

    def AppsDetails(self, event):
        self.installButton.Enable(True)
        self.application = self.appsList.GetItemText(self.appsList.GetSelection())
        self.descriptionFetcher.download(self.application)

    def sizeUpScreen(self, event):
        self.openMin = True

    def WriteApps(self, array):
        self.imagesapps.RemoveAll()

        self.DelApps()
        self.root_apps = self.appsList.AddRoot("")
        self.i = 0
        array.sort(key=unicode.upper)
        for app in array:
            app_array = app.split("~")
            appname = app_array[0]
            try:
                free = int(app_array[3])
                testing = int(app_array[1])
                nocd = int(app_array[2])
            except IndexError:
                free = 0
                testing = 0
                nocd = 0

            show = True
            if nocd == 1 and self.nocdChk.IsChecked() == 0:
                show = False
            if free == 0 and self.commercialChk.IsChecked() == 0:
                show = False
            if testing == 1 and self.testingChk.IsChecked() == 0:
                show = False

            if (show == True):
                self.icon_look_for = Variables.playonlinux_rep + "/configurations/icones/" + appname
                if (os.path.exists(self.icon_look_for.encode('utf-8', 'ignore'))):
                    try:
                        bitmap = wx.Image(self.icon_look_for)
                        bitmap.Rescale(22, 22, wx.IMAGE_QUALITY_HIGH)
                        bitmap = bitmap.ConvertToBitmap()
                        self.imagesapps.Add(bitmap)
                    except:
                        pass
                else:
                    self.imagesapps.Add(wx.Bitmap(Variables.playonlinux_env + "/etc/playonlinux22.png"))
                itemId = self.appsList.AppendItem(self.root_apps, appname.encode('utf-8', 'ignore'), self.i)
                if testing == 1:
                    # (255,255,214) is web site color for beta, but it's not very visible next to plain white,
                    # and red is the color of danger
                    self.appsList.SetItemBackgroundColour(itemId, (255, 214, 214))
                self.i = self.i + 1

    def DelApps(self):
        self.appsList.DeleteAllItems()

    def SetImg(self, image):
        self.image.SetBitmap(wx.Bitmap(image))
        self.Refresh()

    def ResetImg(self):
        self.SetImg(Variables.playonlinux_env + "/resources/images/pol_min.png")

    def CheckBoxReload(self, event):
        chk_id = event.GetId()
        if (chk_id == 401):
            if (self.testingChk.IsChecked() == 1):
                wx.MessageBox(_(
                    "By enabling this option, you can install programs that employ digital rights management (DRM) copy protection that are not supported by {0} and might need to be bypassed.\n\nThis feature should not be construed as implicit or implied condoning of piracy and as such, we will not offer any support for issues resulting from using this option.").format(
                    os.environ["APPLICATION_TITLE"]), _("Attention!"))
        if (chk_id == 402):
            if (self.nocdChk.IsChecked() == 1):
                wx.MessageBox(_(
                    "By enabling this, you will have access to installers for programs that contain protections against copy (DRM) incompatible with emulation.\nThe only workaround is to use \"no-cd\" patches, but since those can also be used for piracy purposes we won't give any support on this matter."),
                    _("Please read this"))

        if (self.searchbox.GetValue() == ""):
            self.AddApps(self, noevent=True)
        else:
            self.search(self)

    def AddApps(self, event, noevent=False):
        self.searchbox.SetValue("")
        if (noevent == False):
            if (event.GetId() >= 3000):
                self.cat_selected = event.GetId() - 3000
            else:
                self.cat_selected = event.GetId() - 2000

            self.current_cat = self.cat_selected
        else:
            try:
                self.cat_selected = self.current_cat
            except:
                return 0
        if (self.cat_selected == 8):
            self.apps = codecs.open(Variables.playonlinux_rep + "/configurations/listes/0", 'r', "utf-8")
        if (self.cat_selected == 3):
            self.apps = codecs.open(Variables.playonlinux_rep + "/configurations/listes/1", 'r', "utf-8")
        if (self.cat_selected == 0):
            self.apps = codecs.open(Variables.playonlinux_rep + "/configurations/listes/2", 'r', "utf-8")
        if (self.cat_selected == 7):
            self.apps = codecs.open(Variables.playonlinux_rep + "/configurations/listes/3", 'r', "utf-8")
        if (self.cat_selected == 5):
            self.apps = codecs.open(Variables.playonlinux_rep + "/configurations/listes/4", 'r', "utf-8")
        if (self.cat_selected == 6):
            self.apps = codecs.open(Variables.playonlinux_rep + "/configurations/listes/5", 'r', "utf-8")
        if (self.cat_selected == 4):
            self.apps = codecs.open(Variables.playonlinux_rep + "/configurations/listes/6", 'r', "utf-8")
        if (self.cat_selected == 1):
            self.apps = codecs.open(Variables.playonlinux_rep + "/configurations/listes/7", 'r', "utf-8")
        if (self.cat_selected == 2):
            self.apps = codecs.open(Variables.playonlinux_rep + "/configurations/listes/8", 'r', "utf-8")
        if (self.cat_selected == 9):
            self.apps = codecs.open(Variables.playonlinux_rep + "/configurations/listes/9", 'r', "utf-8")

        if (self.cat_selected != -1):
            self.apps = self.apps.readlines()
            self.j = 0
            while (self.j < len(self.apps)):
                self.apps[self.j] = self.apps[self.j].replace("\n", "")
                self.j += 1
            self.WriteApps(self.apps)



import json
import os
import threading
import time
import urllib2

from lib import Variables
from lib.playonlinux import keynat
from wine_versions.WineVersionsTools import fetchUserOS

class WineVersionFetcher(threading.Thread):
    def __init__(self, arch="x86"):
        threading.Thread.__init__(self)
        self.thread_message = "#WAIT#"
        self.versions = []
        self.architecture = arch
        self.start()

    def download(self, game):
        self.getDescription = game

    def run(self):
        self.thread_running = True
        while(self.thread_running):
            if(self.thread_message == "get"):
                try :
                    url = os.environ["WINE_SITE"]
                    wfolder = "-".join([fetchUserOS(), self.architecture])


                    req = urllib2.Request(url, None, {'User-Agent': Variables.userAgent})
                    handle = urllib2.urlopen(req, timeout = 5)
                    time.sleep(1)
                    available_distributions = json.loads(handle.read())
                    self.versions_ = []
                    for distribution in available_distributions:
                        if(distribution["name"] == "-".join(["upstream", fetchUserOS(), self.architecture])):
                            for package in distribution["packages"]:
                                version = package["version"]
                                packageUrl = package["url"]
                                sha1sum = package["sha1sum"]
                                if(not os.path.exists(Variables.playonlinux_rep+"/wine/"+wfolder+"/"+version)):
                                    self.versions_.append(version)
                                else:
                                    print("Directory: %s exists" % (Variables.playonlinux_rep+"/wine/"+wfolder+"/"+version))
                        elif(distribution["name"] == "-".join(["staging", fetchUserOS(), self.architecture])):
                            for package in distribution["packages"]:
                                version = package["version"]
                                if(not os.path.exists(Variables.playonlinux_rep+"/wine/"+wfolder+"/"+version+"-staging")):
                                    self.versions_.append(version+"-staging")
                                else:
                                    print("Directory: %s exists" % (Variables.playonlinux_rep+"/wine/"+wfolder+"/"+version+"-staging"))
                        else:
                            print(distribution["name"] + " does not match")

                    self.versions_.sort(key=keynat)
                    self.versions_.reverse()
                    self.versions = self.versions_[:]
                    self.thread_message = "Ok"
                except Exception, e:
                    print(e)
                    time.sleep(1)
                    self.thread_message = "Err"
                    self.versions = ["Wine packages website is unavailable"]

            else:
                time.sleep(0.2)
